# -*- coding: utf-8 -*-

# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

import mock
import os
import sh
import shutil
import stat
import tempfile

from dlrn.config import ConfigOptions
from dlrn import db
from dlrn.drivers.kojidriver import KojiBuildDriver
from dlrn.tests import base
from six.moves import configparser
from time import localtime
from time import strftime


def _mocked_listdir(directory):
    return ['python-pysaml2-3.0-1a.el7.centos.src.rpm']


def _mocked_time():
    return float(1533293385.545039)


def _mocked_call(*args, **kwargs):
    if args[0] == '/usr/bin/git log':
        return '1 2'
    return True


@mock.patch('sh.restorecon', create=True)
@mock.patch('sh.env', create=True)
@mock.patch('os.listdir', side_effect=_mocked_listdir)
class TestDriverKoji(base.TestCase):
    def setUp(self):
        super(TestDriverKoji, self).setUp()
        config = configparser.RawConfigParser()
        config.read("projects.ini")
        config.set("DEFAULT", "build_driver",
                   "dlrn.drivers.kojidriver.KojiBuildDriver")
        self.config = ConfigOptions(config)
        self.config.koji_krb_principal = 'test@example.com'
        self.config.koji_krb_keytab = '/home/test/test.keytab'
        self.config.koji_scratch_build = True
        self.config.koji_build_target = 'build-target'
        self.temp_dir = tempfile.mkdtemp()
        self.config.datadir = self.temp_dir
        # Create fake build log
        with open("%s/kojibuild.log" % self.temp_dir, 'a') as fp:
            fp.write("Created task: 1234")
        # In the rhpkg case, we need to create a full dir structure
        self.rhpkg_extra_dir = "%s/repos/12/34/1234567890abcdef_1_12345678"\
                               % self.temp_dir
        os.makedirs(self.rhpkg_extra_dir)
        with open("%s/rhpkgbuild.log"
                  % self.rhpkg_extra_dir, 'a') as fp:
            fp.write("Created task: 5678")
        # Create a fake rhpkg binary
        with open("%s/rhpkg" % self.temp_dir, 'a') as fp:
            fp.write("true")
        os.chmod("%s/rhpkg" % self.temp_dir,
                 stat.S_IREAD | stat.S_IWRITE | stat.S_IEXEC)
        os.environ['PATH'] = self.temp_dir + ':' + os.environ['PATH']

    def tearDown(self):
        super(TestDriverKoji, self).tearDown()
        shutil.rmtree(self.temp_dir)

    def test_build_package(self, ld_mock, env_mock, rc_mock):
        driver = KojiBuildDriver(cfg_options=self.config)
        driver.build_package(output_directory=self.temp_dir)

        expected = [mock.call(['koji',
                               '--principal', self.config.koji_krb_principal,
                               '--keytab', self.config.koji_krb_keytab,
                               'build', '--wait',
                               self.config.koji_build_target,
                               '%s/python-pysaml2-3.0-1a.el7.centos.src.rpm' %
                               self.temp_dir],
                              _err=driver._process_koji_output,
                              _out=driver._process_koji_output,
                              scratch=True,
                              _cwd=self.temp_dir,
                              _env={'PATH': '/usr/bin/'}),
                    mock.call(['koji', 'download-task', '--logs', '1234'],
                              _err=driver._process_koji_output,
                              _out=driver._process_koji_output,
                              _cwd=self.temp_dir,
                              _env={'PATH': '/usr/bin/'})]
        # 1- koji build (handled by env_mock)
        # 2- koji download (handled by env_mock)
        # 3- restorecon (handled by rc_mock)
        self.assertEqual(env_mock.call_count, 2)
        self.assertEqual(rc_mock.call_count, 1)
        self.assertEqual(env_mock.call_args_list, expected)

    def test_build_package_no_scratch(self, ld_mock, env_mock, rc_mock):
        self.config.koji_scratch_build = False
        driver = KojiBuildDriver(cfg_options=self.config)
        driver.build_package(output_directory=self.temp_dir)

        expected = [mock.call(['koji',
                               '--principal', self.config.koji_krb_principal,
                               '--keytab', self.config.koji_krb_keytab,
                               'build', '--wait',
                               self.config.koji_build_target,
                               '%s/python-pysaml2-3.0-1a.el7.centos.src.rpm' %
                               self.temp_dir],
                              _err=driver._process_koji_output,
                              _out=driver._process_koji_output,
                              scratch=False,
                              _cwd=self.temp_dir,
                              _env={'PATH': '/usr/bin/'}),
                    mock.call(['koji', 'download-task', '--logs', '1234'],
                              _err=driver._process_koji_output,
                              _out=driver._process_koji_output,
                              _cwd=self.temp_dir,
                              _env={'PATH': '/usr/bin/'})]
        # 1- koji build (handled by env_mock)
        # 2- koji download (handled by env_mock)
        # 3- restorecon (handled by rc_mock)
        self.assertEqual(env_mock.call_count, 2)
        self.assertEqual(rc_mock.call_count, 1)
        self.assertEqual(env_mock.call_args_list, expected)

    def test_build_package_brew(self, ld_mock, env_mock, rc_mock):
        self.config.koji_exe = 'brew'
        driver = KojiBuildDriver(cfg_options=self.config)
        driver.build_package(output_directory=self.temp_dir)

        expected = [mock.call(['brew',
                               '--principal', self.config.koji_krb_principal,
                               '--keytab', self.config.koji_krb_keytab,
                               'build', '--wait',
                               self.config.koji_build_target,
                               '%s/python-pysaml2-3.0-1a.el7.centos.src.rpm' %
                               self.temp_dir],
                              _err=driver._process_koji_output,
                              _out=driver._process_koji_output,
                              scratch=True,
                              _cwd=self.temp_dir,
                              _env={'PATH': '/usr/bin/'}),
                    mock.call(['brew', 'download-task', '--logs', '1234'],
                              _err=driver._process_koji_output,
                              _out=driver._process_koji_output,
                              _cwd=self.temp_dir,
                              _env={'PATH': '/usr/bin/'})]
        # 1- koji build (handled by env_mock)
        # 2- koji download (handled by env_mock)
        # 3- restorecon (handled by rc_mock)
        self.assertEqual(env_mock.call_count, 2)
        self.assertEqual(rc_mock.call_count, 1)
        self.assertEqual(env_mock.call_args_list, expected)

    def test_build_package_nokrb(self, ld_mock, env_mock, rc_mock):
        self.config.koji_krb_principal = None
        self.config.koji_krb_keytab = None
        driver = KojiBuildDriver(cfg_options=self.config)
        driver.build_package(output_directory=self.temp_dir)

        expected = [mock.call(['koji',
                               'build', '--wait',
                               self.config.koji_build_target,
                               '%s/python-pysaml2-3.0-1a.el7.centos.src.rpm' %
                               self.temp_dir],
                              _err=driver._process_koji_output,
                              _out=driver._process_koji_output,
                              scratch=True,
                              _cwd=self.temp_dir,
                              _env={'PATH': '/usr/bin/'}),
                    mock.call(['koji', 'download-task', '--logs', '1234'],
                              _err=driver._process_koji_output,
                              _out=driver._process_koji_output,
                              _cwd=self.temp_dir,
                              _env={'PATH': '/usr/bin/'})]
        # 1- koji build (handled by env_mock)
        # 2- koji download (handled by env_mock)
        # 3- restorecon (handled by rc_mock)
        self.assertEqual(env_mock.call_count, 2)
        self.assertEqual(rc_mock.call_count, 1)
        self.assertEqual(env_mock.call_args_list, expected)

    @mock.patch('os.rename')
    @mock.patch.object(sh.Command, '__call__', autospec=True,
                       side_effect=_mocked_call)
    @mock.patch('dlrn.drivers.kojidriver.time', side_effect=_mocked_time)
    @mock.patch('sh.kinit', create=True)
    def test_build_package_rhpkg(self, ki_mock, tm_mock, rh_mock, rn_mock,
                                 ld_mock, env_mock, rc_mock):
        self.config.koji_use_rhpkg = True
        commit = db.Commit(dt_commit=123, project_name='python-pysaml2',
                           commit_hash='1234567890abcdef',
                           distro_hash='1234567890abcdef',
                           extended_hash='1234567890abcdef',
                           dt_distro=123,
                           dt_extended=123)

        driver = KojiBuildDriver(cfg_options=self.config)
        driver.build_package(output_directory=self.temp_dir,
                             package_name='python-pysaml2',
                             commit=commit)

        expected_env = [mock.call(['koji', 'download-task', '--logs', '5678'],
                                  _err=driver._process_koji_output,
                                  _out=driver._process_koji_output,
                                  _cwd=self.rhpkg_extra_dir,
                                  _env={'PATH': '/usr/bin/'})]

        pkg_date = strftime("%Y-%m-%d-%H%M%S", localtime(_mocked_time()))
        expected_rh = [mock.call('%s/rhpkg' % self.temp_dir, 'import',
                                 '--skip-diff',
                                 '%s/python-pysaml2-3.0-1a.el7.centos.src'
                                 '.rpm' % self.temp_dir),
                       mock.call('%s/rhpkg' % self.temp_dir, 'commit', '-p',
                                 '-m',
                                 'DLRN build at %s\n\n'
                                 'Source SHA: 1234567890abcdef\n'
                                 'Dist SHA: 1234567890abcdef\n'
                                 'NVR: python-pysaml2-3.0-1a.el7.centos\n' %
                                 pkg_date),
                       mock.call('/usr/bin/git log', '--pretty=format:%H %ct',
                                 '-1', '.'),
                       mock.call('%s/rhpkg' % self.temp_dir, 'build',
                                 '--skip-nvr-check', scratch=True)]

        # 1- kinit (handled by kb_mock)
        # 2- rhpkg import (handled by rh_mock)
        # 3- rhpkg commit (handled by rh_mock)
        # 4- git log (handled by rh_mock)
        # 5- rename (handled by rn_mock)
        # 5- rhpkg build (handled by rh_mock)
        # 6- koji download (handled by env_mock)
        # 7- restorecon (handled by rc_mock)
        self.assertEqual(ki_mock.call_count, 1)
        self.assertEqual(rh_mock.call_count, 4)
        self.assertEqual(env_mock.call_count, 1)
        self.assertEqual(rc_mock.call_count, 1)
        self.assertEqual(rn_mock.call_count, 1)
        self.assertEqual(env_mock.call_args_list, expected_env)
        self.assertEqual(rh_mock.call_args_list, expected_rh)

    def test_write_mock_config(self, ld_mock, env_mock, rc_mock):
        self.config.koji_build_target = 'foo-target'
        self.config.koji_arch = 'aarch64'
        self.config.fetch_mock_config = True
        self.config.mock_base_packages = 'foo bar'
        driver = KojiBuildDriver(cfg_options=self.config)
        output_file = os.path.join(self.temp_dir, 'dlrn-1.cfg')

        # Create sample downloaded config file
        with open(output_file, "w") as fp:
            fp.write("config_opts['root'] = 'dlrn-centos7-x86_64-1'\n")
            fp.write("config_opts['chroot_setup_cmd'] = 'install abc def\n")
            fp.write("'''")

        expected = "config_opts['chroot_setup_cmd'] = 'install foo bar'\n"

        driver.write_mock_config(output_file)

        with open(output_file, "r") as fp:
            for line in fp.readlines():
                if line.startswith("config_opts['chroot_setup_cmd']"):
                    self.assertEqual(expected, line)

        self.assertEqual(env_mock.call_count, 1)

    def test_write_mock_config_pkg_mgr(self, ld_mock, env_mock, rc_mock):
        self.config.koji_build_target = 'foo-target'
        self.config.koji_arch = 'aarch64'
        self.config.fetch_mock_config = True
        self.config.mock_base_packages = 'foo bar'
        self.config.mock_package_manager = 'apt'
        driver = KojiBuildDriver(cfg_options=self.config)
        output_file = os.path.join(self.temp_dir, 'dlrn-1.cfg')

        # Create sample downloaded config file
        with open(output_file, "w") as fp:
            fp.write("config_opts['root'] = 'dlrn-centos7-x86_64-1'\n")
            fp.write("config_opts['chroot_setup_cmd'] = 'install abc def\n")
            fp.write("'''")

        expected = "config_opts['package_manager'] = 'apt'\n"

        driver.write_mock_config(output_file)

        with open(output_file, "r") as fp:
            for line in fp.readlines():
                if line.startswith("config_opts['package_manager']"):
                    self.assertEqual(expected, line)

        self.assertEqual(env_mock.call_count, 1)

    def test_additional_tags(self, ld_mock, env_mock, rc_mock):
        self.config.koji_add_tags = ['foo', 'bar']
        self.config.koji_exe = 'brew'

        driver = KojiBuildDriver(cfg_options=self.config)
        driver.build_package(output_directory=self.temp_dir)
        expected = [mock.call(['brew',
                               '--principal', self.config.koji_krb_principal,
                               '--keytab', self.config.koji_krb_keytab,
                               'build', '--wait',
                               self.config.koji_build_target,
                               '%s/python-pysaml2-3.0-1a.el7.centos.src.rpm' %
                               self.temp_dir],
                              _err=driver._process_koji_output,
                              _out=driver._process_koji_output,
                              scratch=True,
                              _cwd=self.temp_dir,
                              _env={'PATH': '/usr/bin/'}),
                    mock.call(['brew', 'tag-build', 'foo',
                               'python-pysaml2-3.0-1a.el7.centos'],
                              _err=driver._process_koji_output,
                              _out=driver._process_koji_output,
                              _cwd=self.temp_dir,
                              _env={'PATH': '/usr/bin/'}),
                    mock.call(['brew', 'tag-build', 'bar',
                               'python-pysaml2-3.0-1a.el7.centos'],
                              _err=driver._process_koji_output,
                              _out=driver._process_koji_output,
                              _cwd=self.temp_dir,
                              _env={'PATH': '/usr/bin/'}),
                    mock.call(['brew', 'download-task', '--logs', '1234'],
                              _err=driver._process_koji_output,
                              _out=driver._process_koji_output,
                              _cwd=self.temp_dir,
                              _env={'PATH': '/usr/bin/'})]
        # 1- koji build (handled by env_mock)
        # 2 and 3- koji tag (handled by env_mock)
        # 4- koji download (handled by env_mock)
        # 5- restorecon (handled by rc_mock)
        self.assertEqual(env_mock.call_count, 4)
        self.assertEqual(rc_mock.call_count, 1)
        self.assertEqual(env_mock.call_args_list, expected)



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

"""Rename rpms to artifacts

Revision ID: 2d503b5034b7
Revises: 2a0313a8a7d6
Create Date: 2019-04-26 01:06:50.462042

"""

from alembic import op
import sqlalchemy as sa


# revision identifiers, used by Alembic.
revision = '2d503b5034b7'
down_revision = '2a0313a8a7d6'
branch_labels = None
depends_on = None


def upgrade():
    with op.batch_alter_table("commits") as batch_op:
        batch_op.alter_column('rpms', existing_type=sa.Text(),
                              new_column_name='artifacts')


def downgrade():
    with op.batch_alter_table("commits") as batch_op:
        batch_op.alter_column('artifacts', existing_type=sa.Text(),
                              new_column_name='rpms')



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

# BuildRPMDriver derived classes expose the following methods:
#
# build_package(). This method will perform the actual package build using
#                  the driver-specific approach.

from dlrn.drivers.buildrpm import BuildRPMDriver
import dlrn.shell

import io
import logging
import os
import re
import sh
import shutil

logger = logging.getLogger("dlrn-build-copr")


class CoprBuildDriver(BuildRPMDriver):
    DRIVER_CONFIG = {
        'coprbuild_driver': {
            'coprid': {},
        }
    }

    def __init__(self, *args, **kwargs):
        super(CoprBuildDriver, self).__init__(*args, **kwargs)
        self.exe_name = 'copr'

    # We are using this method to "tee" copr output to a log file and stdout
    def _process_copr_output(self, line):
        if dlrn.shell.verbose_build:
            logger.info(line[:-1])
        self.copr_fp.write(line)

    def build_package(self, **kwargs):
        """Valid parameters:

        :param output_directory: directory where the SRPM is located,
                                 and the built packages will be.
        """
        output_dir = kwargs.get('output_directory')
        coprid = self.config_options.coprid

        # Find src.rpm
        for rpm in os.listdir(output_dir):
            if rpm.endswith(".src.rpm"):
                src_rpm = '%s/%s' % (output_dir, rpm)
        try:
            # Build package using copr
            run_cmd = []
            run_cmd.extend(
                [self.exe_name, 'build',
                 coprid, src_rpm])

            build_exception = None
            with io.open("%s/coprbuild.log" % output_dir, 'a',
                         encoding='utf-8', errors='replace') as self.copr_fp:
                try:
                    sh.env(run_cmd, _err=self._process_copr_output,
                           _out=self._process_copr_output,
                           _cwd=output_dir,
                           _env={'PATH': '/usr/bin/'})
                except Exception as e:
                    build_exception = e

            # Find task id to download logs
            with open("%s/coprbuild.log" % output_dir, 'r') as fp:
                log_content = fp.readlines()
            build_id = None
            for line in log_content:
                m = re.search(r'^Created builds: (\d+)$', line)
                if m:
                    logger.info("Created build id %s" % m.group(1))
                    build_id = m.group(1)
                    break

            if not build_id:
                raise Exception('Failed to find build id for the copr build')

            # Download build artifacts and logs
            ddir = "%s/%s" % (output_dir, build_id)
            run_cmd = []
            run_cmd.extend(
                [self.exe_name, 'download-build', '-d', ddir, build_id])

            with io.open("%s/coprdownload.log" % output_dir, 'a',
                         encoding='utf-8', errors='replace') as self.copr_fp:
                try:
                    sh.env(run_cmd, _err=self._process_copr_output,
                           _out=self._process_copr_output,
                           _cwd=output_dir, _env={'PATH': '/usr/bin/'})
                except Exception as e:
                    raise e

            # Move specific download files in output_dir
            exts_filter = ['.rpm', '.log.gz']
            # Only a directory named with the build target name
            # must be in download directory
            target_name = os.listdir(ddir)[0]
            target_dir = os.path.join(ddir, target_name)
            # Do the copy of file we care of
            for f in os.listdir(target_dir):
                if any([f.endswith(ft) for ft in exts_filter]):
                    src = os.path.join(target_dir, f)
                    dst = os.path.join(output_dir, f)
                    logger.info("Copying %s to %s" % (src, dst))
                    shutil.copy(src, dst)
            # Remove download directory
            logger.info("Removing %s" % ddir)
            shutil.rmtree(ddir)

            # All went fine, create the $OUTPUT_DIRECTORY/installed file
            open('%s/installed' % output_dir, 'a').close()
        finally:
            # Finally run restorecon
            try:
                sh.restorecon('-Rv', output_dir)
            except Exception as e:
                logger.info('restorecon did not run correctly, %s' % e)

            # We only want to raise the build exception at the very end, after
            # downloading all relevant artifacts
            if build_exception:
                raise build_exception



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

import base64
import mock
import os
import sh
import shutil
import tempfile

from datetime import datetime
from dlrn.api import app
from dlrn.config import ConfigOptions
from dlrn import db
from dlrn.tests import base
from dlrn import utils
from flask import json
from six.moves import configparser
from six.moves.urllib.request import urlopen


def mocked_session(url):
    db_fd, filepath = tempfile.mkstemp()
    session = db.getSession("sqlite:///%s" % filepath)
    utils.loadYAML(session, './dlrn/tests/samples/commits_2.yaml')
    return session


def mocked_urlopen(url):
    if url.startswith('http://example.com'):
        fp = open('./dlrn/tests/samples/commits_remote.yaml', 'rb')
        return fp
    else:
        return urlopen(url)


def mock_opt(config_file):
    cp = configparser.RawConfigParser()
    cp.read(config_file)
    co = ConfigOptions(cp)
    co.use_components = True
    return co


def mock_ag(dirname, datadir, session, reponame, hashed_dir=False):
    return 'abc123'


class DLRNAPITestCase(base.TestCase):
    def setUp(self):
        super(DLRNAPITestCase, self).setUp()
        self.db_fd, self.filepath = tempfile.mkstemp()
        app.config['DB_PATH'] = "sqlite:///%s" % self.filepath
        app.config['REPO_PATH'] = '/tmp'
        self.app = app.test_client()
        self.app.testing = True
        self.headers = {'Authorization': 'Basic %s' % (
            base64.b64encode(b'foo:bar').decode('ascii'))}

    def tearDown(self):
        os.close(self.db_fd)
        os.unlink(self.filepath)
        super(DLRNAPITestCase, self).tearDown()


@mock.patch('dlrn.api.dlrn_api.datetime')
@mock.patch('dlrn.api.dlrn_api.getSession', side_effect=mocked_session)
class TestGetLastTestedRepo(DLRNAPITestCase):
    def test_get_last_tested_repo_no_results(self, db_mock, dt_mock):
        req_data = json.dumps(dict(max_age='48'))
        dt_mock.now.return_value = datetime.fromtimestamp(1441901490)
        response = self.app.get('/api/last_tested_repo',
                                data=req_data,
                                content_type='application/json')
        self.assertEqual(response.status_code, 404)

    def test_get_last_tested_repo_no_results_url_params(self, db_mock,
                                                        dt_mock):
        dt_mock.now.return_value = datetime.fromtimestamp(1441901490)
        response = self.app.get('/api/last_tested_repo?max_age=48')
        self.assertEqual(response.status_code, 404)

    def test_get_last_tested_repo_with_age(self, db_mock, dt_mock):
        req_data = json.dumps(dict(max_age='72'))
        dt_mock.now.return_value = datetime.fromtimestamp(1441901490)
        response = self.app.get('/api/last_tested_repo',
                                data=req_data,
                                content_type='application/json')
        data = json.loads(response.data)

        self.assertEqual(data['commit_hash'],
                         '1c67b1ab8c6fe273d4e175a14f0df5d3cbbd0edc')
        self.assertEqual(data['distro_hash'],
                         '8170b8686c38bafb6021d998e2fb268ab26ccf65')
        self.assertEqual(data['job_id'], 'consistent')
        self.assertEqual(data['component'], None)
        self.assertEqual(response.status_code, 200)

    def test_get_last_tested_repo_with_age_url_params(self, db_mock, dt_mock):
        dt_mock.now.return_value = datetime.fromtimestamp(1441901490)
        response = self.app.get('/api/last_tested_repo?max_age=72')
        data = json.loads(response.data)

        self.assertEqual(data['commit_hash'],
                         '1c67b1ab8c6fe273d4e175a14f0df5d3cbbd0edc')
        self.assertEqual(data['distro_hash'],
                         '8170b8686c38bafb6021d998e2fb268ab26ccf65')
        self.assertEqual(data['job_id'], 'consistent')
        self.assertEqual(response.status_code, 200)

    def test_get_last_tested_repo_noconstraints(self, db_mock, dt_mock):
        req_data = json.dumps(dict(max_age='0'))
        dt_mock.now.return_value = datetime.fromtimestamp(1441901490)
        response = self.app.get('/api/last_tested_repo',
                                data=req_data,
                                content_type='application/json')
        data = json.loads(response.data)
        self.assertEqual(data['commit_hash'],
                         '1c67b1ab8c6fe273d4e175a14f0df5d3cbbd0edc')
        self.assertEqual(data['distro_hash'],
                         '8170b8686c38bafb6021d998e2fb268ab26ccf65')
        self.assertEqual(data['job_id'], 'another-ci')
        self.assertEqual(response.status_code, 200)

    def test_get_last_tested_repo_non_existing_ci(self, db_mock, dt_mock):
        req_data = json.dumps(dict(max_age='0', job_id='foo'))
        dt_mock.now.return_value = datetime.fromtimestamp(1441901490)
        response = self.app.get('/api/last_tested_repo',
                                data=req_data,
                                content_type='application/json')
        data = json.loads(response.data)
        self.assertEqual(data['commit_hash'],
                         '1c67b1ab8c6fe273d4e175a14f0df5d3cbbd0edc')
        self.assertEqual(data['distro_hash'],
                         '8170b8686c38bafb6021d998e2fb268ab26ccf65')
        self.assertEqual(data['job_id'], 'consistent')
        self.assertEqual(response.status_code, 200)

    def test_get_last_tested_repo_existing_ci(self, db_mock, dt_mock):
        req_data = json.dumps(dict(max_age='0',
                                   job_id='current-passed-ci'))
        dt_mock.now.return_value = datetime.fromtimestamp(1441901490)
        response = self.app.get('/api/last_tested_repo',
                                data=req_data,
                                content_type='application/json')
        data = json.loads(response.data)
        self.assertEqual(data['commit_hash'],
                         '17234e9ab9dfab4cf5600f67f1d24db5064f1025')
        self.assertEqual(data['distro_hash'],
                         '024e24f0cf4366c2290c22f24e42de714d1addd1')
        self.assertEqual(data['job_id'], 'current-passed-ci')
        self.assertEqual(response.status_code, 200)

    def test_get_last_tested_repo_failed(self, db_mock, dt_mock):
        req_data = json.dumps(dict(max_age='0', success='0'))
        dt_mock.now.return_value = datetime.fromtimestamp(1441901490)
        response = self.app.get('/api/last_tested_repo',
                                data=req_data,
                                content_type='application/json')
        data = json.loads(response.data)
        self.assertEqual(data['commit_hash'],
                         '17234e9ab9dfab4cf5600f67f1d24db5064f1025')
        self.assertEqual(data['distro_hash'],
                         '024e24f0cf4366c2290c22f24e42de714d1addd1')
        self.assertEqual(data['job_id'], 'current-passed-ci')
        self.assertEqual(response.status_code, 200)

    def test_get_last_tested_repo_failed_url_params(self, db_mock, dt_mock):
        dt_mock.now.return_value = datetime.fromtimestamp(1441901490)
        response = self.app.get('/api/last_tested_repo?max_age=0&success=0')
        data = json.loads(response.data)
        self.assertEqual(data['commit_hash'],
                         '17234e9ab9dfab4cf5600f67f1d24db5064f1025')
        self.assertEqual(data['distro_hash'],
                         '024e24f0cf4366c2290c22f24e42de714d1addd1')
        self.assertEqual(data['job_id'], 'current-passed-ci')
        self.assertEqual(response.status_code, 200)

    def test_get_last_tested_repo_sequential(self, db_mock, dt_mock):
        req_data = json.dumps(dict(max_age='0',
                                   job_id='foo-ci',
                                   sequential_mode='true',
                                   previous_job_id='current-passed-ci'))
        dt_mock.now.return_value = datetime.fromtimestamp(1441901490)
        response = self.app.get('/api/last_tested_repo',
                                data=req_data,
                                content_type='application/json')
        data = json.loads(response.data)
        self.assertEqual(data['commit_hash'],
                         '17234e9ab9dfab4cf5600f67f1d24db5064f1025')
        self.assertEqual(data['distro_hash'],
                         '024e24f0cf4366c2290c22f24e42de714d1addd1')
        self.assertEqual(data['job_id'], 'current-passed-ci')
        self.assertEqual(response.status_code, 200)

    def test_get_last_tested_repo_sequential_needs_previous_job_id(self,
                                                                   db_mock,
                                                                   dt_mock):
        req_data = json.dumps(dict(max_age='0',
                                   job_id='foo-ci',
                                   sequential_mode='true'))
        dt_mock.now.return_value = datetime.fromtimestamp(1441901490)
        response = self.app.get('/api/last_tested_repo',
                                data=req_data,
                                content_type='application/json')
        self.assertEqual(response.status_code, 400)

    def test_get_last_tested_repo_sequential_novote(self, db_mock, dt_mock):
        req_data = json.dumps(dict(max_age='0',
                                   job_id='foo-ci',
                                   sequential_mode='true',
                                   previous_job_id='bar-ci'))
        dt_mock.now.return_value = datetime.fromtimestamp(1441901490)
        response = self.app.get('/api/last_tested_repo',
                                data=req_data,
                                content_type='application/json')
        self.assertEqual(response.status_code, 404)

    def test_get_last_tested_repo_component(self, db_mock, dt_mock):
        req_data = json.dumps(dict(max_age='0', success='1',
                                   component='foo-component'))
        dt_mock.now.return_value = datetime.fromtimestamp(1441901490)
        response = self.app.get('/api/last_tested_repo',
                                data=req_data,
                                content_type='application/json')
        data = json.loads(response.data)
        self.assertEqual(data['commit_hash'],
                         '1c67b1ab8c6fe273d4e175a14f0df5d3cbbd0edc')
        self.assertEqual(data['distro_hash'],
                         '8170b8686c38bafb6021d998e2fb268ab26ccf65')
        self.assertEqual(data['component'], 'foo-component')
        self.assertEqual(response.status_code, 200)


@mock.patch('dlrn.api.dlrn_api.datetime')
@mock.patch('dlrn.api.dlrn_api.getSession', side_effect=mocked_session)
@mock.patch('dlrn.api.utils.getSession', side_effect=mocked_session)
class TestPostLastTestedRepo(DLRNAPITestCase):
    def test_post_last_tested_repo_needs_auth(self, db2_mock, db_mock,
                                              dt_mock):
        response = self.app.post('/api/last_tested_repo')
        self.assertEqual(response.status_code, 401)

    def test_post_last_tested_repo_missing_params(self, db2_mock, db_mock,
                                                  dt_mock):
        req_data = json.dumps(dict(max_age='0'))
        dt_mock.now.return_value = datetime.fromtimestamp(1441901490)

        response = self.app.post('/api/last_tested_repo',
                                 data=req_data,
                                 headers=self.headers,
                                 content_type='application/json')
        self.assertEqual(response.status_code, 400)

    def test_post_last_tested_repo_noconstraints(self, db2_mock, db_mock,
                                                 dt_mock):
        req_data = json.dumps(dict(max_age='0',
                                   reporting_job_id='foo-ci'))
        dt_mock.now.return_value = datetime.fromtimestamp(1441901490)

        response = self.app.post('/api/last_tested_repo',
                                 data=req_data,
                                 headers=self.headers,
                                 content_type='application/json')
        data = json.loads(response.data)
        self.assertEqual(response.status_code, 201)
        self.assertEqual(data['commit_hash'],
                         '1c67b1ab8c6fe273d4e175a14f0df5d3cbbd0edc')
        self.assertEqual(data['distro_hash'],
                         '8170b8686c38bafb6021d998e2fb268ab26ccf65')
        self.assertEqual(data['job_id'], 'foo-ci')
        self.assertEqual(data['in_progress'], True)

    def test_post_last_tested_repo_sequential(self, db2_mock, db_mock,
                                              dt_mock):
        req_data = json.dumps(dict(max_age='0',
                                   job_id='foo-ci',
                                   reporting_job_id='foo-ci',
                                   sequential_mode='true',
                                   previous_job_id='current-passed-ci'))
        dt_mock.now.return_value = datetime.fromtimestamp(1441901490)
        response = self.app.post('/api/last_tested_repo',
                                 data=req_data,
                                 headers=self.headers,
                                 content_type='application/json')
        data = json.loads(response.data)
        self.assertEqual(response.status_code, 201)
        self.assertEqual(data['commit_hash'],
                         '17234e9ab9dfab4cf5600f67f1d24db5064f1025')
        self.assertEqual(data['distro_hash'],
                         '024e24f0cf4366c2290c22f24e42de714d1addd1')
        self.assertEqual(data['job_id'], 'foo-ci')
        self.assertEqual(data['in_progress'], True)

    def test_post_last_tested_repo_sequential_needs_previous_job_id(self,
                                                                    db2_mock,
                                                                    db_mock,
                                                                    dt_mock):
        req_data = json.dumps(dict(max_age='0',
                                   job_id='foo-ci',
                                   reporting_job_id='foo-ci',
                                   sequential_mode='true'))
        dt_mock.now.return_value = datetime.fromtimestamp(1441901490)
        response = self.app.post('/api/last_tested_repo',
                                 data=req_data,
                                 headers=self.headers,
                                 content_type='application/json')
        self.assertEqual(response.status_code, 400)

    def test_post_last_tested_repo_sequential_novote(self, db2_mock, db_mock,
                                                     dt_mock):
        req_data = json.dumps(dict(max_age='0',
                                   job_id='foo-ci',
                                   reporting_job_id='foo-ci',
                                   sequential_mode='true',
                                   previous_job_id='bar-ci'))
        dt_mock.now.return_value = datetime.fromtimestamp(1441901490)
        response = self.app.post('/api/last_tested_repo',
                                 data=req_data,
                                 headers=self.headers,
                                 content_type='application/json')
        self.assertEqual(response.status_code, 404)

    def test_post_last_tested_repo_component(self, db2_mock, db_mock,
                                             dt_mock):
        req_data = json.dumps(dict(max_age='0',
                                   reporting_job_id='foo-ci',
                                   component='foo-component'))
        dt_mock.now.return_value = datetime.fromtimestamp(1441901490)

        response = self.app.post('/api/last_tested_repo',
                                 data=req_data,
                                 headers=self.headers,
                                 content_type='application/json')
        data = json.loads(response.data)
        self.assertEqual(response.status_code, 201)
        self.assertEqual(data['commit_hash'],
                         '1c67b1ab8c6fe273d4e175a14f0df5d3cbbd0edc')
        self.assertEqual(data['distro_hash'],
                         '8170b8686c38bafb6021d998e2fb268ab26ccf65')
        self.assertEqual(data['job_id'], 'foo-ci')
        self.assertEqual(data['in_progress'], True)
        self.assertEqual(data['component'], 'foo-component')


@mock.patch('dlrn.api.dlrn_api.getSession', side_effect=mocked_session)
@mock.patch('dlrn.api.utils.getSession', side_effect=mocked_session)
class TestReportResult(DLRNAPITestCase):
    def test_report_result_needs_auth(self, db2_mock, db_mock):
        response = self.app.post('/api/report_result')
        self.assertEqual(response.status_code, 401)

    def test_report_result_missing_params(self, db2_mock, db_mock):
        req_data = json.dumps(dict(commit_hash='0'))

        response = self.app.post('/api/report_result',
                                 data=req_data,
                                 headers=self.headers,
                                 content_type='application/json')
        self.assertEqual(response.status_code, 400)

    def test_report_result_wrong_commit(self, db2_mock, db_mock):
        req_data = json.dumps(dict(commit_hash='0', job_id='foo-ci', url='',
                                   timestamp='1941635095', success='true',
                                   distro_hash='8170b8686c38bafb6021d998e2fb26'
                                               '8ab26ccf65'))

        response = self.app.post('/api/report_result',
                                 data=req_data,
                                 headers=self.headers,
                                 content_type='application/json')
        self.assertEqual(response.status_code, 404)

    def test_report_result_no_commit_hash(self, db2_mock, db_mock):
        req_data = json.dumps(dict(distro_hash='0', job_id='foo-ci', url='',
                                   timestamp='1941635095', success='true'))

        response = self.app.post('/api/report_result',
                                 data=req_data,
                                 headers=self.headers,
                                 content_type='application/json')
        self.assertEqual(response.status_code, 400)

    def test_report_result_no_distro_hash(self, db2_mock, db_mock):
        req_data = json.dumps(dict(commit_hash='0', job_id='foo-ci', url='',
                                   timestamp='1941635095', success='true'))

        response = self.app.post('/api/report_result',
                                 data=req_data,
                                 headers=self.headers,
                                 content_type='application/json')
        self.assertEqual(response.status_code, 400)

    def test_report_result_successful(self, db2_mock, db_mock):
        req_data = json.dumps(dict(commit_hash='1c67b1ab8c6fe273d4e'
                                               '175a14f0df5d3cbbd0edc',
                                   job_id='foo-ci', url='',
                                   timestamp='1941635095', success='true',
                                   distro_hash='8170b8686c38bafb6021'
                                               'd998e2fb268ab26ccf65'))

        response = self.app.post('/api/report_result',
                                 data=req_data,
                                 headers=self.headers,
                                 content_type='application/json')
        data = json.loads(response.data)
        self.assertEqual(response.status_code, 201)
        self.assertEqual(data['in_progress'], False)
        self.assertEqual(data['success'], True)

    def test_report_result_aggregate(self, db2_mock, db_mock):
        req_data = json.dumps(dict(aggregate_hash='12345678',
                                   job_id='phase2-ci', url='',
                                   timestamp='1941635095', success='true'))

        response = self.app.post('/api/report_result',
                                 data=req_data,
                                 headers=self.headers,
                                 content_type='application/json')
        data = json.loads(response.data)
        self.assertEqual(response.status_code, 201)
        self.assertEqual(data['in_progress'], False)
        self.assertEqual(data['success'], True)
        self.assertEqual(data['aggregate_hash'], '12345678')

    def test_report_result_agg_and_hash(self, db2_mock, db_mock):
        req_data = json.dumps(dict(aggregate_hash='12345678',
                                   job_id='phase2-ci', url='',
                                   timestamp='1941635095', success='true',
                                   commit_hash='123456'))
        response = self.app.post('/api/report_result',
                                 data=req_data,
                                 headers=self.headers,
                                 content_type='application/json')
        self.assertEqual(response.status_code, 400)


@mock.patch('dlrn.api.dlrn_api.getSession', side_effect=mocked_session)
@mock.patch('dlrn.api.utils.getSession', side_effect=mocked_session)
class TestPromote(DLRNAPITestCase):
    def test_promote_needs_auth(self, db2_mock, db_mock):
        response = self.app.post('/api/promote')
        self.assertEqual(response.status_code, 401)

    def test_promote_missing_commit(self, db2_mock, db_mock):
        req_data = json.dumps(dict(commit_hash='abc123', distro_hash='abc123',
                                   promote_name='foo'))
        response = self.app.post('/api/promote',
                                 data=req_data,
                                 headers=self.headers,
                                 content_type='application/json')

        self.assertEqual(response.status_code, 404)

    def test_promote_purged_commit(self, db2_mock, db_mock):
        req_data = json.dumps(dict(commit_hash='1c67b1ab8c6fe273d4e17'
                                               '5a14f0df5d3cbbd0e77',
                                   distro_hash='8170b8686c38bafb6021d'
                                               '998e2fb268ab26ccf77',
                                   promote_name='foo-ci'))
        response = self.app.post('/api/promote',
                                 data=req_data,
                                 headers=self.headers,
                                 content_type='application/json')
        data = json.loads(response.data)

        self.assertEqual(response.status_code, 410)
        self.assertEqual(data['message'], 'commit_hash+distro_hash has been'
                                          ' purged, cannot promote it')

    @mock.patch('os.symlink')
    def test_promote_successful(self, sl_mock, db2_mock, db_mock):
        req_data = json.dumps(dict(commit_hash='1c67b1ab8c6fe273d4e'
                                               '175a14f0df5d3cbbd0edc',
                                   distro_hash='8170b8686c38bafb6021'
                                               'd998e2fb268ab26ccf65',
                                   promote_name='foo-ci'))
        response = self.app.post('/api/promote',
                                 data=req_data,
                                 headers=self.headers,
                                 content_type='application/json')

        expected = [mock.call('1c/67/1c67b1ab8c6fe273d4e175a'
                              '14f0df5d3cbbd0edc_8170b868', '/tmp/foo-ci')]

        self.assertEqual(response.status_code, 201)
        self.assertEqual(sl_mock.call_args_list, expected)

    def test_promote_invalid_name(self, db2_mock, db_mock):
        req_data = json.dumps(dict(commit_hash='1c67b1ab8c6fe273d4e'
                                               '175a14f0df5d3cbbd0edc',
                                   distro_hash='8170b8686c38bafb6021'
                                               'd998e2fb268ab26ccf65',
                                   promote_name='consistent'))
        response = self.app.post('/api/promote',
                                 data=req_data,
                                 headers=self.headers,
                                 content_type='application/json')

        self.assertEqual(response.status_code, 403)

    def test_promote_invalid_name_relative(self, db2_mock, db_mock):
        req_data = json.dumps(dict(commit_hash='1c67b1ab8c6fe273d4e'
                                               '175a14f0df5d3cbbd0edc',
                                   distro_hash='8170b8686c38bafb6021'
                                               'd998e2fb268ab26ccf65',
                                   promote_name='../consistent'))
        response = self.app.post('/api/promote',
                                 data=req_data,
                                 headers=self.headers,
                                 content_type='application/json')

        self.assertEqual(response.status_code, 403)


@mock.patch('dlrn.api.dlrn_api.getSession', side_effect=mocked_session)
@mock.patch('dlrn.api.utils.getSession', side_effect=mocked_session)
class TestPromoteBatch(DLRNAPITestCase):
    def setUp(self):
        super(TestPromoteBatch, self).setUp()
        if os.path.exists('/tmp/component/None'):
            shutil.rmtree('/tmp/component/None')
        if os.path.exists('/tmp/component/tripleo'):
            shutil.rmtree('/tmp/component/tripleo')
        os.makedirs('/tmp/component/None')
        os.makedirs('/tmp/component/tripleo')

    def tearDown(self):
        if os.path.exists('/tmp/component/None'):
            shutil.rmtree('/tmp/component/None')
        if os.path.exists('/tmp/component/tripleo'):
            shutil.rmtree('/tmp/component/tripleo')
        super(TestPromoteBatch, self).tearDown()

    def test_promote_batch_needs_auth(self, db2_mock, db_mock):
        response = self.app.post('/api/promote-batch')
        self.assertEqual(response.status_code, 401)

    def test_promote_batch_missing_commit(self, db2_mock, db_mock):
        req_data = json.dumps([dict(commit_hash='abc123', distro_hash='abc123',
                                    promote_name='foo'),
                               dict(commit_hash='abc456', distro_hash='abc456',
                                    promote_name='foo')])
        response = self.app.post('/api/promote-batch',
                                 data=req_data,
                                 headers=self.headers,
                                 content_type='application/json')
        self.assertEqual(response.status_code, 404)

    @mock.patch('os.symlink')
    def test_promote_batch_successful_1(self, sl_mock, db2_mock, db_mock):
        req_data = json.dumps([dict(commit_hash='1c67b1ab8c6fe273d4e'
                                                '175a14f0df5d3cbbd0edc',
                                    distro_hash='8170b8686c38bafb6021'
                                                'd998e2fb268ab26ccf65',
                                    promote_name='foo-ci')])
        response = self.app.post('/api/promote-batch',
                                 data=req_data,
                                 headers=self.headers,
                                 content_type='application/json')

        expected = [mock.call('1c/67/1c67b1ab8c6fe273d4e175a'
                              '14f0df5d3cbbd0edc_8170b868', '/tmp/foo-ci')]

        self.assertEqual(response.status_code, 201)
        self.assertEqual(sl_mock.call_args_list, expected)

    @mock.patch('os.symlink')
    def test_promote_batch_successful_2(self, sl_mock, db2_mock, db_mock):
        req_data = json.dumps([dict(commit_hash='1c67b1ab8c6fe273d4e'
                                                '175a14f0df5d3cbbd0edc',
                                    distro_hash='8170b8686c38bafb6021'
                                                'd998e2fb268ab26ccf65',
                                    promote_name='foo-ci'),
                               dict(commit_hash='17234e9ab9dfab4cf560'
                                                '0f67f1d24db5064f1025',
                                    distro_hash='024e24f0cf4366c2290c'
                                                '22f24e42de714d1addd1',
                                    promote_name='foo-ci'),
                               ])
        response = self.app.post('/api/promote-batch',
                                 data=req_data,
                                 headers=self.headers,
                                 content_type='application/json')

        expected = [mock.call('1c/67/1c67b1ab8c6fe273d4e175a'
                              '14f0df5d3cbbd0edc_8170b868', '/tmp/foo-ci'),
                    mock.call('component/tripleo/17/23/17234e9ab9dfab4cf56'
                              '00f67f1d24db5064f1025_024e24f0', '/tmp/foo-ci')]
        self.assertEqual(response.status_code, 201)
        self.assertEqual(sl_mock.call_args_list, expected)

    @mock.patch('dlrn.api.dlrn_api.aggregate_repo_files', side_effect=mock_ag)
    @mock.patch('dlrn.api.dlrn_api._get_config_options', side_effect=mock_opt)
    @mock.patch('os.symlink')
    def test_promote_batch_successful_2_cmp(self, sl_mock, co_mock, ag_mock,
                                            db2_mock, db_mock):
        req_data = json.dumps([dict(commit_hash='1c67b1ab8c6fe273d4e'
                                                '175a14f0df5d3cbbd0edc',
                                    distro_hash='8170b8686c38bafb6021'
                                                'd998e2fb268ab26ccf65',
                                    promote_name='foo-ci'),
                               dict(commit_hash='17234e9ab9dfab4cf560'
                                                '0f67f1d24db5064f1025',
                                    distro_hash='024e24f0cf4366c2290c'
                                                '22f24e42de714d1addd1',
                                    promote_name='foo-ci')])
        response = self.app.post('/api/promote-batch',
                                 data=req_data,
                                 headers=self.headers,
                                 content_type='application/json')

        expected = [mock.call('1c/67/1c67b1ab8c6fe273d4e175a'
                              '14f0df5d3cbbd0edc_8170b868',
                              '/tmp/component/None/foo-ci'),
                    mock.call('17/23/17234e9ab9dfab4cf5600f67f1d24db5064'
                              'f1025_024e24f0',
                              '/tmp/component/tripleo/foo-ci')]

        data = json.loads(response.data)
        self.assertEqual(response.status_code, 201)
        self.assertEqual(data['commit_hash'], '17234e9ab9dfab4cf560'
                                              '0f67f1d24db5064f1025')
        self.assertEqual(data['distro_hash'], '024e24f0cf4366c2290c'
                                              '22f24e42de714d1addd1')
        self.assertEqual(data['promote_name'], 'foo-ci')
        self.assertEqual(data['component'], 'tripleo')
        self.assertEqual(data['aggregate_hash'], 'abc123')
        self.assertEqual(data['user'], 'foo')
        self.assertEqual(sl_mock.call_args_list, expected)
        self.assertEqual(ag_mock.call_count, 1)


@mock.patch('dlrn.api.dlrn_api.getSession', side_effect=mocked_session)
@mock.patch('dlrn.api.utils.getSession', side_effect=mocked_session)
class TestRepoStatus(DLRNAPITestCase):
    def test_repo_status_missing_commit(self, db2_mock, db_mock):
        req_data = json.dumps(dict(commit_hash='abc123', distro_hash='abc123'))
        response = self.app.get('/api/repo_status',
                                data=req_data,
                                content_type='application/json')

        self.assertEqual(response.status_code, 404)

    def test_repo_status_missing_commit_url_param(self, db2_mock, db_mock):
        response = self.app.get('/api/repo_status?commit_hash=abc123'
                                '&distro_hash=abc123')
        self.assertEqual(response.status_code, 404)

    def test_repo_status_multiple_votes(self, db2_mock, db_mock):
        req_data = json.dumps(dict(commit_hash='17234e9ab9dfab4cf5600f'
                                               '67f1d24db5064f1025',
                                   distro_hash='024e24f0cf4366c2290c22'
                                               'f24e42de714d1addd1'))
        response = self.app.get('/api/repo_status',
                                data=req_data,
                                content_type='application/json')

        self.assertEqual(response.status_code, 200)
        data = json.loads(response.data)
        self.assertEqual(len(data), 2)

    def test_repo_status_multiple_votes_url_param(self, db2_mock, db_mock):
        response = self.app.get('/api/repo_status?commit_hash='
                                '17234e9ab9dfab4cf5600f67f1d24db5064f1025&'
                                'distro_hash=024e24f0cf4366c2290c22f24e42d'
                                'e714d1addd1')
        self.assertEqual(response.status_code, 200)
        data = json.loads(response.data)
        self.assertEqual(len(data), 2)

    def test_repo_status_with_success(self, db2_mock, db_mock):
        req_data = json.dumps(dict(commit_hash='17234e9ab9dfab4cf5600f'
                                               '67f1d24db5064f1025',
                                   distro_hash='024e24f0cf4366c2290c22'
                                               'f24e42de714d1addd1',
                                   success='true'))
        response = self.app.get('/api/repo_status',
                                data=req_data,
                                content_type='application/json')

        self.assertEqual(response.status_code, 200)
        data = json.loads(response.data)
        self.assertEqual(len(data), 1)

    def test_repo_status_with_success_url_param(self, db2_mock, db_mock):
        response = self.app.get('/api/repo_status?commit_hash='
                                '17234e9ab9dfab4cf5600f67f1d24db5064f1025&'
                                'distro_hash=024e24f0cf4366c2290c22f24e42d'
                                'e714d1addd1&success=true')
        self.assertEqual(response.status_code, 200)
        data = json.loads(response.data)
        self.assertEqual(len(data), 1)


@mock.patch('dlrn.api.dlrn_api.getSession', side_effect=mocked_session)
@mock.patch('dlrn.api.utils.getSession', side_effect=mocked_session)
class TestAggStatus(DLRNAPITestCase):
    def test_agg_status_missing_hash(self, db2_mock, db_mock):
        response = self.app.get('/api/agg_status?success=true')
        self.assertEqual(response.status_code, 400)

    def test_agg_status_multiple_votes(self, db2_mock, db_mock):
        response = self.app.get('/api/agg_status?aggregate_hash=12345678')
        self.assertEqual(response.status_code, 200)
        data = json.loads(response.data)
        self.assertEqual(len(data), 2)

    def test_agg_status_with_success(self, db2_mock, db_mock):
        response = self.app.get('/api/agg_status?aggregate_hash=12345678&'
                                'success=true')
        self.assertEqual(response.status_code, 200)
        data = json.loads(response.data)
        self.assertEqual(len(data), 1)

    def test_agg_status_hash_not_found(self, db2_mock, db_mock):
        response = self.app.get('/api/agg_status?aggregate_hash=000000000')
        self.assertEqual(response.status_code, 200)
        data = json.loads(response.data)
        self.assertEqual(len(data), 0)


@mock.patch('dlrn.remote.getSession', side_effect=mocked_session)
@mock.patch('dlrn.api.utils.getSession', side_effect=mocked_session)
class TestRemoteImport(DLRNAPITestCase):
    def test_post_remote_import_needs_auth(self, db2_mock, db_mock):
        response = self.app.post('/api/remote/import')
        self.assertEqual(response.status_code, 401)

    @mock.patch('os.rename')
    @mock.patch('os.symlink')
    @mock.patch('dlrn.drivers.rdoinfo.RdoInfoDriver.getpackages')
    @mock.patch.object(sh.Command, '__call__', autospec=True)
    @mock.patch('dlrn.remote.post_build')
    @mock.patch('dlrn.remote.urlopen', side_effect=mocked_urlopen)
    def test_post_remote_import_success(self, url_mock, build_mock, sh_mock,
                                        db2_mock, db_mock, gp_mock, sl_mock,
                                        rn_mock):

        req_data = json.dumps(dict(repo_url='http://example.com/1/'))

        header = {'Authorization': 'Basic %s' % (
                  base64.b64encode(b'foo:bar').decode('ascii'))}

        response = self.app.post('/api/remote/import',
                                 data=req_data,
                                 headers=header,
                                 content_type='application/json')

        self.assertEqual(response.status_code, 201)
        data = json.loads(response.data)
        self.assertEqual(data['repo_url'], 'http://example.com/1/')


@mock.patch('dlrn.api.dlrn_api.render_template', side_effect=' ')
@mock.patch('dlrn.api.dlrn_api.getSession', side_effect=mocked_session)
@mock.patch('dlrn.api.utils.getSession', side_effect=mocked_session)
class TestGetCIVotes(DLRNAPITestCase):
    def test_get_civotes(self, db2_mock, db_mock, rt_mock):
        response = self.app.get('/api/civotes.html')
        self.assertEqual(rt_mock.call_count, 1)
        self.assertEqual(response.status_code, 200)

    def test_get_civotes_detail_fail(self, db2_mock, db_mock, rt_mock):
        response = self.app.get('/api/civotes_detail.html')
        self.assertEqual(response.status_code, 400)

    def test_get_civotes_detail_ok(self, db2_mock, db_mock, rt_mock):
        response = self.app.get('/api/civotes_detail.html?'
                                'commit_hash=17234e9ab9dfab4cf5600f67f1d24db5'
                                '064f1025&distro_hash=024e24f0cf4366c2290c22f'
                                '24e42de714d1addd1')
        self.assertEqual(rt_mock.call_count, 1)
        self.assertEqual(response.status_code, 200)

    def test_get_civotes_detail_with_ci(self, db2_mock, db_mock, rt_mock):
        response = self.app.get('/api/civotes_detail.html?'
                                'ci_name=another-ci')
        self.assertEqual(rt_mock.call_count, 1)
        self.assertEqual(response.status_code, 200)


@mock.patch('dlrn.api.dlrn_api.render_template', side_effect=' ')
@mock.patch('dlrn.api.dlrn_api.getSession', side_effect=mocked_session)
@mock.patch('dlrn.api.utils.getSession', side_effect=mocked_session)
class TestGetCIAggVotes(DLRNAPITestCase):
    def test_get_ciaggvotes(self, db2_mock, db_mock, rt_mock):
        response = self.app.get('/api/civotes_agg.html')
        self.assertEqual(rt_mock.call_count, 1)
        self.assertEqual(response.status_code, 200)

    def test_get_ciaggvotes_detail_fail(self, db2_mock, db_mock, rt_mock):
        response = self.app.get('/api/civotes_agg_detail.html')
        self.assertEqual(response.status_code, 400)

    def test_get_ciaggvotes_detail_ok(self, db2_mock, db_mock, rt_mock):
        response = self.app.get('/api/civotes_agg_detail.html?'
                                'ref_hash=12345678')
        self.assertEqual(rt_mock.call_count, 1)
        self.assertEqual(response.status_code, 200)

    def test_get_ciaggvotes_detail_with_ci(self, db2_mock, db_mock, rt_mock):
        response = self.app.get('/api/civotes_agg_detail.html?'
                                'ci_name=phase2-ci')
        self.assertEqual(rt_mock.call_count, 1)
        self.assertEqual(response.status_code, 200)


@mock.patch('dlrn.api.dlrn_api.render_template', side_effect=' ')
@mock.patch('dlrn.api.dlrn_api.getSession', side_effect=mocked_session)
@mock.patch('dlrn.api.utils.getSession', side_effect=mocked_session)
class TestGetReport(DLRNAPITestCase):

    @mock.patch('dlrn.db.Commit.getshardedcommitdir')
    def test_get_report(self, commit_mock, db2_mock, db_mock, rt_mock):
        response = self.app.get('/api/report.html')
        self.assertEqual(rt_mock.call_count, 1)
        self.assertEqual(response.status_code, 200)
        assert commit_mock.called

    @mock.patch('dlrn.db.Commit.getshardedcommitdir')
    def test_get_report_detail(self, commit_mock, db2_mock, db_mock, rt_mock):
        response = self.app.get('/api/report.html?package=python-pysaml2'
                                '&success=1')
        self.assertEqual(1, rt_mock.call_args.kwargs['count'])
        self.assertEqual(response.status_code, 200)
        assert commit_mock.called

    def test_get_report_detail_wrong_query(self, db2_mock, db_mock, rt_mock):
        response = self.app.get('/api/report.html?project=python-pysaml2'
                                '&success=1')
        self.assertTrue(rt_mock.call_args.kwargs['count'] != 1)
        self.assertEqual(response.status_code, 200)

    def test_get_package_version(self, db2_mock, db_mock, rt_mock):
        response = self.app.get('/api/report.html')
        key = ('93eee77657978547f5fad1cb8cd30b570da83e68_'
               '008678d7b0e20fbae185f2bb1bd0d9d167586211_None')
        package_info = rt_mock.call_args.kwargs['commits_build_dir'].get(key)
        self.assertEqual('6.1.1', package_info.get('version'))
        self.assertEqual('0.20200421095106.5aa891c.el7',
                         package_info.get('release'))
        self.assertEqual(response.status_code, 200)

    def test_get_package_version_wrong_version(self, db2_mock, db_mock,
                                               rt_mock):
        response = self.app.get('/api/report.html')
        key = ('1c67b1ab8c6fe273d4e175a14f0df5d3cbbd0edc_'
               '8170b8686c38bafb6021d998e2fb268ab26ccf65_None')
        package_info = rt_mock.call_args.kwargs['commits_build_dir'].get(key)
        self.assertEqual('0.8.3-0.99.20150907.2338git.el7.centos.src.rpm',
                         package_info.get('version'))
        self.assertEqual('', package_info.get('release'))
        self.assertEqual(response.status_code, 200)


@mock.patch('dlrn.api.dlrn_api.getSession', side_effect=mocked_session)
@mock.patch('dlrn.api.utils.getSession', side_effect=mocked_session)
class TestGetPromotions(DLRNAPITestCase):
    def test_get_promotions_missing_commit(self, db2_mock, db_mock):
        req_data = json.dumps(dict(distro_hash='abc123'))
        response = self.app.get('/api/promotions',
                                data=req_data,
                                content_type='application/json')
        self.assertEqual(response.status_code, 400)

    def test_get_promotions_no_such_commit(self, db2_mock, db_mock):
        req_data = json.dumps(dict(commit_hash='abc123',
                              distro_hash='abc123'))
        response = self.app.get('/api/promotions',
                                data=req_data,
                                content_type='application/json')
        self.assertEqual(response.status_code, 404)

    def test_get_promotions_no_such_commit_url_params(self, db2_mock,
                                                      db_mock):
        response = self.app.get('/api/promotions?commit_hash=abc123&'
                                'distro_hash=abc123')
        self.assertEqual(response.status_code, 404)

    def test_get_promotions_multiple_votes(self, db2_mock, db_mock):
        req_data = '{}'
        response = self.app.get('/api/promotions',
                                data=req_data,
                                content_type='application/json')
        self.assertEqual(response.status_code, 200)
        data = json.loads(response.data)
        self.assertEqual(len(data), 3)

    def test_get_promotions_multiple_votes_url_params(self, db2_mock, db_mock):
        response = self.app.get('/api/promotions')
        self.assertEqual(response.status_code, 200)
        data = json.loads(response.data)
        self.assertEqual(len(data), 3)

    def test_get_promotions_with_promote_name(self, db2_mock, db_mock):
        req_data = json.dumps(dict(promote_name='another-ci'))
        response = self.app.get('/api/promotions',
                                data=req_data,
                                content_type='application/json')
        self.assertEqual(response.status_code, 200)
        data = json.loads(response.data)
        self.assertEqual(len(data), 1)

    def test_get_promotions_with_promote_name_url_params(self, db2_mock,
                                                         db_mock):
        response = self.app.get('/api/promotions?promote_name=another-ci')
        self.assertEqual(response.status_code, 200)
        data = json.loads(response.data)
        self.assertEqual(len(data), 1)

    def test_get_promotions_with_commit(self, db2_mock, db_mock):
        req_data = json.dumps(dict(commit_hash='17234e9ab9dfab4cf5600f67f1'
                                               'd24db5064f1025',
                                   distro_hash='024e24f0cf4366c2290c22f24e'
                                               '42de714d1addd1'))
        response = self.app.get('/api/promotions',
                                data=req_data,
                                content_type='application/json')
        self.assertEqual(response.status_code, 200)
        data = json.loads(response.data)
        self.assertEqual(len(data), 1)

    def test_get_promotions_with_offset(self, db2_mock, db_mock):
        req_data = json.dumps(dict(offset=1))
        response = self.app.get('/api/promotions',
                                data=req_data,
                                content_type='application/json')
        self.assertEqual(response.status_code, 200)
        data = json.loads(response.data)
        self.assertEqual(len(data), 2)

    def test_get_promotions_with_offset_url_params(self, db2_mock, db_mock):
        response = self.app.get('/api/promotions?offset=1')
        self.assertEqual(response.status_code, 200)
        data = json.loads(response.data)
        self.assertEqual(len(data), 2)

    def test_get_promotions_with_limit(self, db2_mock, db_mock):
        req_data = json.dumps(dict(limit=1))
        response = self.app.get('/api/promotions',
                                data=req_data,
                                content_type='application/json')
        self.assertEqual(response.status_code, 200)
        data = json.loads(response.data)
        self.assertEqual(len(data), 1)

    def test_get_promotions_with_limit_url_param(self, db2_mock, db_mock):
        response = self.app.get('/api/promotions?limit=1')
        self.assertEqual(response.status_code, 200)
        data = json.loads(response.data)
        self.assertEqual(len(data), 1)

    def test_get_promotions_same_timestamp(self, db2_mock, db_mock):
        response = self.app.get('/api/promotions?promote_name=foo-ci&limit=2')
        self.assertEqual(response.status_code, 200)
        data = json.loads(response.data)
        self.assertEqual(len(data), 2)
        self.assertEqual(data[0]['aggregate_hash'], 'abcd1234')
        # This proves that the second commit, with the same timestamp, is the
        # one with a lower id
        self.assertEqual(data[1]['aggregate_hash'], None)


@mock.patch('dlrn.api.dlrn_api.getSession', side_effect=mocked_session)
@mock.patch('dlrn.api.utils.getSession', side_effect=mocked_session)
class TestMetrics(DLRNAPITestCase):
    def test_metrics_missing_start_date(self, db2_mock, db_mock):
        req_data = json.dumps(dict(end_date='0'))
        response = self.app.get('/api/metrics/builds',
                                data=req_data,
                                content_type='application/json')
        self.assertEqual(response.status_code, 400)

    def test_metrics_missing_start_date_url_param(self, db2_mock, db_mock):
        response = self.app.get('/api/metrics/builds?end_date=0')
        self.assertEqual(response.status_code, 400)

    def test_metrics_wrong_date_format(self, db2_mock, db_mock):
        req_data = json.dumps(dict(start_date='0', end_date='0'))
        response = self.app.get('/api/metrics/builds',
                                data=req_data,
                                content_type='application/json')
        self.assertEqual(response.status_code, 400)

    def test_metrics_wrong_date_format_url_param(self, db2_mock, db_mock):
        response = self.app.get('/api/metrics/builds?start_date=0&end_date=0')
        self.assertEqual(response.status_code, 400)

    def test_metrics_notindate(self, db2_mock, db_mock):
        req_data = json.dumps(dict(start_date='2011-09-07',
                                   end_date='2011-09-09'))
        response = self.app.get('/api/metrics/builds',
                                data=req_data,
                                content_type='application/json')
        self.assertEqual(response.status_code, 200)
        data = json.loads(response.data)
        self.assertEqual(data['total'], 0)
        self.assertEqual(data['succeeded'], 0)
        self.assertEqual(data['failed'], 0)

    def test_metrics_success(self, db2_mock, db_mock):
        req_data = json.dumps(dict(start_date='2015-09-07',
                                   end_date='2015-09-09'))
        response = self.app.get('/api/metrics/builds',
                                data=req_data,
                                content_type='application/json')
        self.assertEqual(response.status_code, 200)
        data = json.loads(response.data)
        self.assertEqual(data['total'], 4)
        self.assertEqual(data['succeeded'], 4)
        self.assertEqual(data['failed'], 0)

    def test_metrics_success_url_param(self, db2_mock, db_mock):
        response = self.app.get('/api/metrics/builds?start_date=2015-09-07'
                                '&end_date=2015-09-09')
        self.assertEqual(response.status_code, 200)
        data = json.loads(response.data)
        self.assertEqual(data['total'], 4)
        self.assertEqual(data['succeeded'], 4)
        self.assertEqual(data['failed'], 0)

    def test_metrics_filtered(self, db2_mock, db_mock):
        req_data = json.dumps(dict(start_date='2015-09-07',
                                   end_date='2015-09-09',
                                   package_name='python-pysaml2'))
        response = self.app.get('/api/metrics/builds',
                                data=req_data,
                                content_type='application/json')
        self.assertEqual(response.status_code, 200)
        data = json.loads(response.data)
        self.assertEqual(data['total'], 1)
        self.assertEqual(data['succeeded'], 1)
        self.assertEqual(data['failed'], 0)

    def test_metrics_filtered_url_param(self, db2_mock, db_mock):
        response = self.app.get('/api/metrics/builds?start_date=2015-09-07'
                                '&end_date=2015-09-09&'
                                'package_name=python-pysaml2')
        self.assertEqual(response.status_code, 200)
        data = json.loads(response.data)
        self.assertEqual(data['total'], 1)
        self.assertEqual(data['succeeded'], 1)
        self.assertEqual(data['failed'], 0)

    def test_metrics_filtered_nopackage(self, db2_mock, db_mock):
        req_data = json.dumps(dict(start_date='2015-09-07',
                                   end_date='2015-09-09',
                                   package_name='python-pysaml'))
        response = self.app.get('/api/metrics/builds',
                                data=req_data,
                                content_type='application/json')
        self.assertEqual(response.status_code, 200)
        data = json.loads(response.data)
        self.assertEqual(data['total'], 0)

    def test_metrics_filtered_nopackage_url_param(self, db2_mock, db_mock):
        response = self.app.get('/api/metrics/builds?start_date=2015-09-07'
                                '&end_date=2015-09-09&'
                                'package_name=python-pysaml')
        self.assertEqual(response.status_code, 200)
        data = json.loads(response.data)
        self.assertEqual(data['total'], 0)



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

import mock
import os
import shutil
import tempfile

from dlrn.config import ConfigOptions
from dlrn import db
from dlrn import rsync
from dlrn.tests import base
from six.moves import configparser


@mock.patch('sh.rsync', create=True)
class TestSyncRepo(base.TestCase):
    def setUp(self):
        super(TestSyncRepo, self).setUp()
        config = configparser.RawConfigParser()
        config.read("projects.ini")
        config.set('DEFAULT', 'datadir', tempfile.mkdtemp())
        config.set('DEFAULT', 'scriptsdir', tempfile.mkdtemp())
        config.set('DEFAULT', 'baseurl', "file://%s" % config.get('DEFAULT',
                                                                  'datadir'))
        config.set('DEFAULT', 'rsyncport', '30000')
        config.set('DEFAULT', 'rsyncdest', 'user@host:/directory')

        self.config = ConfigOptions(config)
        self.commit = db.Commit(dt_commit=123, project_name='foo',
                                commit_hash='1c67b1ab8c6fe273d4e175a14f0df5'
                                            'd3cbbd0edf',
                                repo_dir='/home/dlrn/data/foo',
                                distro_hash='c31d1b18eb5ab5aed6721fc4fad06c9'
                                            'bd242490f',
                                dt_distro=123,
                                distgit_dir='/home/dlrn/data/foo_distro',
                                commit_branch='master', dt_build=1441245153)

    def tearDown(self):
        super(TestSyncRepo, self).tearDown()
        shutil.rmtree(self.config.datadir)
        shutil.rmtree(self.config.scriptsdir)

    def test_sync_repo(self, sh_mock):
        yumdir = os.path.join(self.config.datadir, 'repos', '.',
                              self.commit.getshardedcommitdir())
        repodir = os.path.join(self.config.datadir, 'repos', '.')

        rsync.sync_repo(self.commit)
        expected = [mock.call('-avzR', '--delete-delay', '-e',
                              'ssh -p 30000 -o StrictHostKeyChecking=no',
                              [yumdir,
                               os.path.join(repodir, 'report.html'),
                               os.path.join(repodir, 'status_report.html'),
                               os.path.join(repodir, 'styles.css'),
                               os.path.join(repodir, 'queue.html'),
                               os.path.join(repodir, 'status_report.csv')],
                              'user@host:/directory')]
        self.assertEqual(sh_mock.call_args_list, expected)

    def test_sync_repo_component(self, sh_mock):
        self.commit.component = 'foocomp'
        self.config.use_components = True
        yumdir = os.path.join(self.config.datadir, 'repos', '.',
                              self.commit.getshardedcommitdir())
        repodir = os.path.join(self.config.datadir, 'repos', '.')

        rsync.sync_repo(self.commit)
        expected = [mock.call('-avzR', '--delete-delay', '-e',
                              'ssh -p 30000 -o StrictHostKeyChecking=no',
                              [yumdir,
                               os.path.join(repodir, 'report.html'),
                               os.path.join(repodir, 'status_report.html'),
                               os.path.join(repodir, 'styles.css'),
                               os.path.join(repodir, 'queue.html'),
                               os.path.join(repodir, 'status_report.csv')],
                              'user@host:/directory')]
        self.assertEqual(sh_mock.call_args_list, expected)

    def test_sync_symlinks(self, sh_mock):
        repodir = os.path.join(self.config.datadir, 'repos', '.')

        rsync.sync_symlinks(self.commit)
        expected = [mock.call('-avzR', '--delete-delay', '-e',
                              'ssh -p 30000 -o StrictHostKeyChecking=no',
                              [os.path.join(repodir, 'consistent'),
                               os.path.join(repodir, 'current')],
                              'user@host:/directory')]
        self.assertEqual(sh_mock.call_args_list, expected)

    def test_sync_symlinks_component(self, sh_mock):
        self.commit.component = 'foocomp'
        self.config.use_components = True
        repodir = os.path.join(self.config.datadir, 'repos', '.')

        rsync.sync_symlinks(self.commit)
        expected = [mock.call('-avzR', '--delete-delay', '-e',
                              'ssh -p 30000 -o StrictHostKeyChecking=no',
                              ['--exclude', 'consistent/delorean.repo',
                               '--exclude', 'consistent/delorean.repo.md5',
                               '--exclude', 'consistent/versions.csv',
                               '--exclude', 'current/delorean.repo',
                               '--exclude', 'current/delorean.repo.md5',
                               '--exclude', 'current/versions.csv'],
                              [os.path.join(repodir, 'component/foocomp',
                                            'consistent'),
                               os.path.join(repodir, 'component/foocomp',
                                            'current'),
                               os.path.join(repodir, 'consistent'),
                               os.path.join(repodir, 'current')],
                              'user@host:/directory'),
                    mock.call('-avzR', '--delete-delay', '-e',
                              'ssh -p 30000 -o StrictHostKeyChecking=no',
                              ['%s/delorean.repo' %
                               os.path.join(repodir, 'consistent'),
                               '%s/delorean.repo.md5' %
                               os.path.join(repodir, 'consistent'),
                               '%s/versions.csv' %
                               os.path.join(repodir, 'consistent'),
                               '%s/delorean.repo' %
                               os.path.join(repodir, 'current'),
                               '%s/delorean.repo.md5' %
                               os.path.join(repodir, 'current'),
                               '%s/versions.csv' %
                               os.path.join(repodir, 'current')],
                              'user@host:/directory')]
        self.assertEqual(sh_mock.call_args_list, expected)



# -*- coding: utf-8 -*-

# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

import mock
import os
import shutil
import sys
import tempfile
import time

from dlrn.tests import base

from datetime import datetime
from dlrn import db
from dlrn import purge
from dlrn import utils
from six.moves import configparser

expected_repos = [
    './data/repos/ae/9d/ae9d27e5100f002f55ad6eb2b252a0aa5f16a336_024e24f0',
    './data/repos/45/95/459549c9ab7fef91b2dc8986bc0643bb2f6ec0c8_885e8077',
    './data/repos/e4/f7/e4f71ada86ee4a42287cf401b77d48c9f98ca5aa_354991d9',
    './data/repos/80/f6/80f6969520b526b01c210b1c680565bef8a1f8ad_024e24f0',
    './data/repos/12/ed/12edde8a53c7299105bb8297c6f31d3a458615ab_354991d9',
    './data/repos/17/23/17234e9ab9dfab4cf5600f67f1d24db5064f1025_024e24f0']


def mocked_session(url):
    db_fd, filepath = tempfile.mkstemp()
    session = db.getSession("sqlite:///%s" % filepath)
    utils.loadYAML(session, './dlrn/tests/samples/commits_1.yaml')
    return session


def mocked_session_2(url):
    db_fd, filepath = tempfile.mkstemp()
    session = db.getSession("sqlite:///%s" % filepath)
    utils.loadYAML(session, './dlrn/tests/samples/commits_2.yaml')
    return session


def mocked_listdir(path):
    return ['file1.rpm', 'file2.rpm', 'file3.rpm']


def mocked_islink(path):
    return False


def mocked_is_commit_in_dirs(commit, dirlist, basedir, component_list=None):
    # We are making one of the commit hashes be in the excluded dir list
    if commit.commit_hash == '6abf557aa1d8fff0aa21f8eba6cd18302c2c86ff':
        return True
    return False


def mocked_exists_false(path):
    return False


@mock.patch('os.path.islink', side_effect=mocked_islink)
@mock.patch('os.listdir', side_effect=mocked_listdir)
@mock.patch('dlrn.purge.getSession', side_effect=mocked_session)
@mock.patch('shutil.rmtree')
@mock.patch('dlrn.purge.is_commit_in_dirs',
            side_effect=mocked_is_commit_in_dirs)
class TestPurge(base.TestCase):
    @mock.patch('dlrn.purge.datetime')
    def test_purge(self, dt_mock, icid_mock, sh_mock, db_mock, lst_mock,
                   il_mock):
        testargs = ["dlrn-purge", "--config-file",
                    "projects.ini", "--older-than",
                    "1", "-y"]
        dt_mock.now.return_value = datetime(2015, 10, 1, 14, 20)
        with mock.patch.object(sys, 'argv', testargs):
            purge.purge()
            expected = []
            for repo in expected_repos:
                expected.append(mock.call(repo, ignore_errors=True))
            self.assertEqual(sh_mock.call_args_list, expected)


@mock.patch('os.path.exists', side_effect=mocked_exists_false)
class TestIsCommitInDirs(base.TestCase):
    def test_is_commit_in_dirs_component(self, ex_mock):
        commit = db.Commit(dt_commit=123, project_name='foo', type="rpm",
                           component='bar',
                           commit_hash='1c67b1ab8c6fe273d4e175a14f0df5'
                                       'd3cbbd0edf',
                           repo_dir='/home/dlrn/data/foo',
                           distro_hash='c31d1b18eb5ab5aed6721fc4fad06c9'
                                       'bd242490f',
                           dt_distro=123,
                           distgit_dir='/home/dlrn/data/foo_distro',
                           commit_branch='master', dt_build=1441245153,
                           artifacts='foo-1.0.0.rpm')

        dirlist = ('/home/dlrn/data/repos/consistent,'
                   '/home/dlrn/data/repos/foo-ci')
        basedir = '/home/dlrn/data/repos/'
        expected = [mock.call('/home/dlrn/data/repos/consistent/'
                              'foo-1.0.0.rpm'),
                    mock.call('/home/dlrn/data/repos/component/bar/consistent/'
                              'foo-1.0.0.rpm'),
                    mock.call('/home/dlrn/data/repos/foo-ci/'
                              'foo-1.0.0.rpm'),
                    mock.call('/home/dlrn/data/repos/component/bar/foo-ci/'
                              'foo-1.0.0.rpm')]

        purge.is_commit_in_dirs(commit, dirlist, basedir,
                                component_list=['bar'])

        self.assertEqual(ex_mock.call_args_list, expected)


@mock.patch('shutil.rmtree')
@mock.patch('dlrn.purge.getSession', side_effect=mocked_session_2)
class TestAggPurge(base.TestCase):
    def setUp(self):
        super(TestAggPurge, self).setUp()
        self.config = configparser.RawConfigParser()
        self.config.read("projects.ini")
        self.temp_dir = tempfile.mkdtemp()
        self.config.set('DEFAULT', 'datadir', self.temp_dir)
        os.makedirs(os.path.join(self.temp_dir, 'repos',
                                 'another-ci/12/34/12345678'))
        os.makedirs(os.path.join(self.temp_dir, 'repos',
                                 'foo-ci/90/ab/90abcdef'))

    def tearDown(self):
        super(TestAggPurge, self).tearDown()
        shutil.rmtree(self.temp_dir)

    def test_purge_promoted(self, db_mock, rm_mock):
        purge.purge_promoted_hashes(self.config, time.time(), dry_run=False)
        self.assertEqual(rm_mock.call_count, 2)

    def test_purge_promoted_only_one(self, db_mock, rm_mock):
        current_time = time.time()
        time.sleep(1)
        # Update modification time on one of the dirs
        os.utime(os.path.join(self.temp_dir, 'repos',
                              'foo-ci/90/ab/90abcdef'), None)
        purge.purge_promoted_hashes(self.config, current_time, dry_run=False)
        self.assertEqual(rm_mock.call_count, 1)

    def test_purge_promoted_dry_run(self, db_mock, rm_mock):
        purge.purge_promoted_hashes(self.config, time.time(), dry_run=True)
        self.assertEqual(rm_mock.call_count, 0)

    def test_purge_promoted_protected_path(self, db_mock, rm_mock):
        orig_path = os.path.join(self.temp_dir, 'repos',
                                 'foo-ci/90/ab/90abcdef', 'delorean.repo')
        dst_path = os.path.join(self.temp_dir, 'repos', 'foo-ci/delorean.repo')
        os.symlink(orig_path, dst_path)
        purge.purge_promoted_hashes(self.config, time.time(), dry_run=False)
        self.assertEqual(rm_mock.call_count, 1)



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

"""Scheme change due to SCM support moving to a plugin

Revision ID: 47ebe0522809
Revises: o3c62b0d3ec34
Create Date: 2016-06-08 11:29:16.460075

"""

from alembic import op
import sqlalchemy as sa
from sqlalchemy.sql import table, column


# revision identifiers, used by Alembic.
revision = '47ebe0522809'
down_revision = '3c62b0d3ec34'
branch_labels = None
depends_on = None


def upgrade():
    op.add_column('commits', sa.Column('distgit_dir', sa.String()))
    commits = table('commits',
                    column('id', sa.Integer),
                    column('repo_dir', sa.String),
                    column('distgit_dir', sa.String))
    op.execute(commits.update()
               .where(commits.c.id == commits.c.id)
               .values(distgit_dir=(commits.c.repo_dir + "_distro")))


def downgrade():
    op.drop_column('commits', 'distgit_dir')



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

"""Add component to commits

Revision ID: f84aca0549fd
Revises: b6f658f481f8
Create Date: 2019-09-24 11:46:13.682176

"""

from alembic import op
import sqlalchemy as sa
from sqlalchemy.sql import table, column


# revision identifiers, used by Alembic.
revision = 'f84aca0549fd'
down_revision = 'b6f658f481f8'
branch_labels = None
depends_on = None


def upgrade():
    op.add_column('commits', sa.Column('component', sa.String(64)))
    commits = table('commits',
                    column('id', sa.Integer),
                    column('component', sa.String))
    # For existing commits, set component to None
    op.execute(commits.update()
               .where(commits.c.id == commits.c.id)
               .values(component=None))


def downgrade():
    with op.batch_alter_table('commits') as batch_op:
        batch_op.drop_column('component')



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

"""Add type column

Revision ID: b6f658f481f8
Revises: 2d503b5034b7
Create Date: 2019-04-26 07:29:09.071710

"""

from alembic import op
import sqlalchemy as sa
from sqlalchemy.sql import table, column


# revision identifiers, used by Alembic.
revision = 'b6f658f481f8'
down_revision = '2d503b5034b7'
branch_labels = None
depends_on = None


def upgrade():
    op.add_column("commits", sa.Column("type", sa.String(18)))
    commits = table('commits', column('type', sa.String(18)))
    op.execute(commits.update().values(type="rpm"))


def downgrade():
    with op.batch_alter_table('commits') as batch_op:
        batch_op.drop_column('type')



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.


import argparse
import logging
import os
import sys

from six.moves import configparser
from six.moves import urllib
from six.moves.urllib.request import urlopen
from tempfile import mkstemp

from dlrn.config import ConfigOptions
from dlrn.config import setup_logging
from dlrn.db import closeSession
from dlrn.db import getLastProcessedCommit
from dlrn.db import getSession
from dlrn.shell import post_build
from dlrn.shell import process_build_result
from dlrn.utils import import_object
from dlrn.utils import loadYAML_list
from dlrn.utils import lock_file


logger = logging.getLogger("dlrn-remote")


def import_commit(repo_url, config_file, db_connection=None,
                  local_info_repo=None):
    cp = configparser.RawConfigParser()
    cp.read(config_file)
    config_options = ConfigOptions(cp)
    pkginfo_driver = config_options.pkginfo_driver
    pkginfo = import_object(pkginfo_driver, cfg_options=config_options)
    packages = pkginfo.getpackages(local_info_repo=local_info_repo,
                                   tags=config_options.tags,
                                   dev_mode=False)

    remote_yaml = repo_url + '/' + 'commit.yaml'
    r = urlopen(remote_yaml)
    contents = map(lambda x: x.decode('utf8'), r.readlines())

    osfd, tmpfilename = mkstemp()
    fp = os.fdopen(osfd, 'w')
    fp.writelines(contents)
    fp.close()

    commits = loadYAML_list(tmpfilename)
    os.remove(tmpfilename)
    datadir = os.path.realpath(config_options.datadir)
    if not os.path.exists(datadir):
        os.makedirs(datadir)

    for commit in commits:
        commit.id = None
        if commit.artifacts == 'None':
            commit.artifacts = None
        commit.dt_build = int(commit.dt_build)
        commit.dt_commit = float(commit.dt_commit)
        commit.dt_distro = int(commit.dt_distro)
        # Check if the latest built commit for this project is newer
        # than this one. In that case, we should ignore it
        if db_connection:
            session = getSession(db_connection)
        else:
            session = getSession(config_options.database_connection)
        package = commit.project_name
        old_commit = getLastProcessedCommit(session, package)
        if old_commit:
            if old_commit.dt_commit >= commit.dt_commit:
                if old_commit.dt_distro >= commit.dt_distro:
                    logger.info('Skipping commit %s, a newer commit is '
                                'already built\n'
                                'Old: %s %s, new: %s %s' %
                                (commit.commit_hash, old_commit.dt_commit,
                                 old_commit.dt_distro, commit.dt_commit,
                                 commit.dt_distro))
                    continue    # Skip

        yumrepodir = os.path.join(datadir, "repos",
                                  commit.getshardedcommitdir())
        if not os.path.exists(yumrepodir):
            os.makedirs(yumrepodir)

        for logfile in ['build.log', 'installed', 'mock.log', 'root.log',
                        'rpmbuild.log', 'state.log']:
            logfile_url = repo_url + '/' + logfile
            try:
                r = urlopen(logfile_url)
                contents = map(lambda x: x.decode('utf8'), r.readlines())
                with open(os.path.join(yumrepodir, logfile), "w") as fp:
                    fp.writelines(contents)
            except urllib.error.HTTPError:
                # Ignore errors, if the remote build failed there may be
                # some missing files
                pass

        if commit.artifacts:
            for rpm in commit.artifacts.split(","):
                rpm_url = repo_url + '/' + rpm.split('/')[-1]
                try:
                    r = urlopen(rpm_url)
                    contents = r.read()
                    with open(os.path.join(datadir, rpm), "wb") as fp:
                        fp.write(contents)
                except urllib.error.HTTPError:
                    if rpm != 'None':
                        logger.warning("Failed to download rpm file %s"
                                       % rpm_url)
        # Get remote update lock, to prevent any other remote operation
        # while we are creating the repo and updating the database
        logger.debug("Acquiring remote update lock")
        with lock_file(os.path.join(datadir, 'remote.lck')):
            logger.debug("Acquired lock")
            if commit.status == 'SUCCESS':
                built_rpms = []
                for rpm in commit.artifacts.split(","):
                    built_rpms.append(rpm)
                status = [commit, built_rpms, commit.notes, None]
                post_build(status, packages, session)
            else:
                pkg = [p for p in packages if p['name'] == package][0]
                # Here we fire a refresh of the repositories
                # (upstream and distgit) to be sure to have them in the
                # data directory. We need that in the case the worker
                # is running on another host mainly for the
                # submit_review.sh script.
                pkginfo.getinfo(project=pkg["name"], package=pkg,
                                since='-1', local=False, dev_mode=False)
                # Paths on the worker might differ so we overwrite them
                # to reflect data path on the local API host.
                commit.distgit_dir = pkginfo.distgit_dir(pkg['name'])
                commit.repo_dir = os.path.join(
                    config_options.datadir, pkg['name'])
                status = [commit, '', '', commit.notes]
            process_build_result(status, packages, session, [])
            closeSession(session)   # Keep one session per commit
        logger.debug("Released lock")
    return 0


def remote():
    parser = argparse.ArgumentParser()
    # Some of the non-positional arguments are required, so change the text
    # saying "optional arguments" to just "arguments":
    parser._optionals.title = 'arguments'

    parser.add_argument('--config-file',
                        default='projects.ini',
                        help="Config file. Default: projects.ini")
    parser.add_argument('--repo-url',
                        help="Base repository URL for remotely generated repo "
                             "(required)", required=True)
    parser.add_argument('--info-repo',
                        help="use a local rdoinfo repo instead of "
                             "fetching the default one using rdopkg. Only "
                             "applies when pkginfo_driver is rdoinfo in "
                             "projects.ini")
    parser.add_argument('--debug', action='store_true',
                        help="Print debug logs")

    options = parser.parse_args(sys.argv[1:])

    setup_logging(options.debug)

    return import_commit(options.repo_url, options.config_file,
                         local_info_repo=options.info_repo)



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

"""Add promotions table

Revision ID: 4a5651777e5e
Revises: 638f980c9169
Create Date: 2017-08-21 11:09:46.349973

"""

from alembic import op
import sqlalchemy as sa


# revision identifiers, used by Alembic.
revision = '4a5651777e5e'
down_revision = '638f980c9169'
branch_labels = None
depends_on = None


def upgrade():
    op.create_table('promotions',
                    sa.Column('id', sa.Integer(), nullable=False),
                    sa.Column('commit_id', sa.Integer(),
                              sa.ForeignKey('commits.id'),
                              nullable=False),
                    sa.Column('promotion_name', sa.String(256),
                              nullable=False),
                    sa.Column('timestamp', sa.Integer(), nullable=False),
                    sa.PrimaryKeyConstraint('id'))


def downgrade():
    op.drop_table('promotions')



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

import dlrn.shell
import filecmp
import logging
import multiprocessing
import os
import sh
import shutil

from six.moves.urllib.request import urlopen

from dlrn.config import getConfigOptions
from dlrn.utils import import_object
from time import time

logger = logging.getLogger("dlrn-build")


def _get_yumrepodir(commit):
    return os.path.join("repos", commit.getshardedcommitdir())


def build_worker(packages, commit, run_cmd=False, build_env=None,
                 dev_mode=False, use_public=False, order=False,
                 sequential=False):
    config_options = getConfigOptions()

    if run_cmd:
        try:
            run(run_cmd, commit, build_env, dev_mode, use_public, order,
                do_build=False)
            return [commit, '', '', None]
        except Exception as e:
            return [commit, '', '', e]

    if config_options.use_components:
        logger.info("Processing %s %s for component %s" %
                    (commit.project_name, commit.commit_hash,
                     commit.component))
    else:
        logger.info("Processing %s %s" % (commit.project_name,
                                          commit.commit_hash))

    notes = ""
    try:
        built_rpms, notes = build(packages, commit, build_env, dev_mode,
                                  use_public, order, sequential)
        return [commit, built_rpms, notes, None]
    except Exception as e:
        return [commit, '', '', e]


def get_version_from(packages, project_name):
    for package in packages:
        if package['name'] == project_name:
            return package.get('version-from')
    return None


def build(packages, commit, env_vars, dev_mode, use_public,
          bootstrap, sequential):
    if commit.type == "rpm":
        return build_rpm(
            packages, commit, env_vars, dev_mode, use_public,
            bootstrap, sequential)
    elif commit.type == "container":
        return build_container(
            packages, commit, env_vars, dev_mode, use_public,
            bootstrap, sequential)
    else:
        raise Exception("Unknown type %s" % commit.type)


def build_container(packages, commit, env_vars, dev_mode, use_public,
                    bootstrap, sequential):
    raise NotImplementedError()


def build_rpm(packages, commit, env_vars, dev_mode, use_public,
              bootstrap, sequential):
    config_options = getConfigOptions()
    # Set the build timestamp to now
    commit.dt_build = int(time())

    project_name = commit.project_name
    datadir = os.path.realpath(config_options.datadir)
    yumrepodir = _get_yumrepodir(commit)
    version_from = get_version_from(packages, project_name)

    try:
        build_rpm_wrapper(commit, dev_mode, use_public, bootstrap,
                          env_vars, sequential,
                          version_from=version_from)
    except Exception as e:
        logger.error('Build failed. See logs at: %s/%s/' % (datadir,
                                                            yumrepodir))
        raise Exception("Error in build_rpm_wrapper for %s: %s" %
                        (project_name, e))

    # This *could* have changed during the build, see kojidriver.py
    yumrepodir = _get_yumrepodir(commit)
    yumrepodir_abs = os.path.join(datadir, yumrepodir)

    built_rpms = []
    for rpm in os.listdir(yumrepodir_abs):
        if rpm.endswith(".rpm"):
            built_rpms.append(os.path.join(yumrepodir, rpm))
    if not built_rpms:
        raise Exception("No rpms built for %s" % project_name)

    notes = "OK"

    if not os.path.isfile(os.path.join(yumrepodir_abs, "installed")):
        logger.error('Build failed. See logs at: %s/%s/' % (datadir,
                                                            yumrepodir))
        raise Exception("Error installing %s" % project_name)
    else:
        # Overwrite installed file, adding the repo reference
        with open(os.path.join(yumrepodir_abs, "installed"), "w") as fp:
            fp.write("%s %s %s" % (commit.project_name,
                                   commit.commit_hash,
                                   commit.distro_hash))
    return built_rpms, notes


def build_rpm_wrapper(commit, dev_mode, use_public, bootstrap, env_vars,
                      sequential, version_from=None):
    config_options = getConfigOptions()
    # Get the worker id
    if sequential is True:
        worker_id = 1
    else:
        worker_id = multiprocessing.current_process()._identity[0]

    # Retrieve build driver
    build_driver = config_options.build_driver
    buildrpm = import_object(build_driver, cfg_options=config_options)

    # FIXME(hguemar): move all the mock config logic to driver
    mock_config = "dlrn-" + str(worker_id) + ".cfg"
    scriptsdir = os.path.realpath(config_options.scriptsdir)
    configdir = os.path.realpath(config_options.configdir)
    datadir = os.path.realpath(config_options.datadir)
    baseurl = config_options.baseurl
    templatecfg = os.path.join(configdir, config_options.target + ".cfg")
    newcfg = os.path.join(datadir, mock_config + ".new")
    oldcfg = os.path.join(datadir, mock_config)
    shutil.copyfile(templatecfg, newcfg)

    if (config_options.build_driver ==
            'dlrn.drivers.kojidriver.KojiBuildDriver' and
            config_options.fetch_mock_config):
        buildrpm.write_mock_config(oldcfg)

    # Add the most current repo, we may have dependencies in it
    current_repo = os.path.join(datadir, "repos", "current",
                                "%s.repo" % config_options.reponame)
    if os.path.exists(current_repo):
        # Read the .repo file
        with open(current_repo) as fp:
            current_repo_contents = fp.readlines()
        with open(newcfg, "r") as fp:
            contents = fp.readlines()
        # delete the last line which must be """
        contents = contents[:-1]
        contents = contents + current_repo_contents + ["\"\"\""]
        with open(newcfg, "w") as fp:
            fp.writelines(contents)

    # Set the worker id in the mock configuration, to allow multiple workers
    # for the same config
    with open(newcfg, "r") as fp:
        contents = fp.readlines()
    with open(newcfg, "w") as fp:
        for line in contents:
            if line.startswith("config_opts['root']"):
                line = line[:-2] + "-" + str(worker_id) + "'\n"
            fp.write(line)

    # delete the last line which must be """
    with open(newcfg, "r") as fp:
        contents = fp.readlines()
    contents = contents[:-1]

    try:
        if not baseurl:
            raise Exception("No baseurl defined")
        r = urlopen(baseurl + "/delorean-deps.repo")
        delorean_deps = True
    except Exception:
        logger.warning(
            "Could not open %s/delorean-deps.repo. If some dependent"
            " repositories must be included in the mock then check the"
            " baseurl value in projects.ini, and make sure the file can be"
            " downloaded." % baseurl)
        delorean_deps = False

    if delorean_deps:
        contents.extend(map(lambda x: x.decode('utf8'), r.readlines()))
        contents = contents + ["\n\"\"\""]
        with open(newcfg, "w") as fp:
            fp.writelines(contents)

    if dev_mode or use_public:
        with open(newcfg, "r") as fp:
            contents = fp.readlines()

        # delete the last line which must be """
        contents = contents[:-1]
        try:
            r = urlopen(baseurl + "/current/delorean.repo")
        except Exception as e:
            logger.error("Could not open %s/current/delorean.repo. Check the "
                         "baseurl value in projects.ini, and make sure the "
                         "file can be downloaded." % baseurl)
            raise e

        contents.extend(map(lambda x: x.decode('utf8'), r.readlines()))
        contents.extend(["\n\"\"\""])

        with open(newcfg, "w") as fp:
            fp.writelines(contents)

    # don't change dlrn.cfg if the content hasn't changed to prevent
    # mock from rebuilding its cache.
    try:
        if not filecmp.cmp(newcfg, oldcfg):
            if (config_options.build_driver ==
                    'dlrn.drivers.kojidriver.KojiBuildDriver'):
                if not config_options.fetch_mock_config:
                    shutil.copyfile(newcfg, oldcfg)
            else:
                shutil.copyfile(newcfg, oldcfg)
    except OSError:
        shutil.copyfile(newcfg, oldcfg)

    # Set env variable for Copr configuration
    if (config_options.build_driver ==
            'dlrn.drivers.coprdriver.CoprBuildDriver' and
            config_options.coprid):
        os.environ['COPR_ID'] = config_options.coprid

    # Set release numbering option
    if config_options.release_numbering == '0.1.date.hash':
        os.environ['RELEASE_NUMBERING'] = '0.1.date.hash'
    elif config_options.release_numbering == 'minor.date.hash':
        os.environ['RELEASE_NUMBERING'] = 'minor.date.hash'
        os.environ['RELEASE_MINOR'] = config_options.release_minor
    else:
        os.environ['RELEASE_NUMBERING'] = '0.date.hash'

    # Set env variable for mock configuration
    os.environ['MOCK_CONFIG'] = mock_config

    # if bootstraping, set the appropriate mock config option
    if bootstrap is True:
        additional_mock_options = '-D repo_bootstrap 1'
    else:
        additional_mock_options = None

    dlrn.shell.pkginfo.preprocess(package_name=commit.project_name,
                                  commit_hash=commit.commit_hash)

    if (config_options.pkginfo_driver ==
            'dlrn.drivers.gitrepo.GitRepoDriver' and
            config_options.keep_tarball):
        if (commit.commit_branch == config_options.source or
                commit.commit_branch == 'master'):
            # We are following the master tarball here, use it
            os.environ['DLRN_KEEP_TARBALL'] = '1'
        else:
            if 'DLRN_KEEP_TARBALL' in os.environ:
                del os.environ['DLRN_KEEP_TARBALL']

    if config_options.keep_changelog:
        os.environ['DLRN_KEEP_CHANGELOG'] = '1'

    if (config_options.pkginfo_driver == 'dlrn.drivers.local.LocalDriver'):
        os.environ['DLRN_KEEP_SPEC_AS_IS'] = '1'

    # We may do some git repo manipulation, so we need to make sure the
    # right commit is there
    os.environ['DLRN_SOURCE_COMMIT'] = commit.commit_hash

    run(os.path.join(scriptsdir, "build_srpm.sh"), commit, env_vars,
        dev_mode, use_public, bootstrap, version_from=version_from)

    # SRPM is built, now build the RPM using the driver
    datadir = os.path.realpath(config_options.datadir)
    yumrepodir = _get_yumrepodir(commit)
    yumrepodir_abs = os.path.join(datadir, yumrepodir)

    # If we are using the downstream driver, write the reference commit
    if (config_options.pkginfo_driver ==
            'dlrn.drivers.downstream.DownstreamInfoDriver'):
        dlrn.shell.pkginfo._write_reference_commit(yumrepodir_abs)

    buildrpm.build_package(output_directory=yumrepodir_abs,
                           additional_mock_opts=additional_mock_options,
                           package_name=commit.project_name,
                           commit=commit)


def run(program, commit, env_vars, dev_mode, use_public, bootstrap,
        do_build=True, version_from=None):
    config_options = getConfigOptions()
    datadir = os.path.realpath(config_options.datadir)
    yumrepodir = _get_yumrepodir(commit)
    yumrepodir_abs = os.path.join(datadir, yumrepodir)
    project_name = commit.project_name
    repo_dir = commit.repo_dir

    if do_build:
        # If yum repo already exists remove it and assume we're starting fresh
        if os.path.exists(yumrepodir_abs):
            shutil.rmtree(yumrepodir_abs)
        os.makedirs(yumrepodir_abs)

    if version_from:
        logger.info('Taking tags to define version from %s' % version_from)
        git = sh.git.bake(_cwd=repo_dir, _tty_out=False)
        git.merge('-s', 'ours', '-m', '"fake merge tags"', version_from)

    run_cmd = []
    if env_vars:
        for env_var in env_vars:
            run_cmd.append(env_var)

    run_cmd.extend([program,
                    config_options.target, project_name,
                    os.path.join(datadir, yumrepodir),
                    datadir, config_options.baseurl,
                    os.path.realpath(commit.distgit_dir)])
    if not do_build:
        logger.info('Running %s' % ' '.join(run_cmd))

    try:
        sh.env(run_cmd, _err=process_mock_output, _out=process_mock_output)
    except Exception as e:
        # This *could* have changed during the build, see kojidriver.py
        datadir = os.path.realpath(config_options.datadir)
        yumrepodir = _get_yumrepodir(commit)
        logger.error('cmd failed. See logs at: %s/%s/' % (datadir,
                                                          yumrepodir))
        raise e


def process_mock_output(line):
    if dlrn.shell.verbose_build:
        logger.info(line[:-1])



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

import argparse
import glob
import logging
import os
import shutil
import sys

from datetime import datetime
from datetime import timedelta
from six.moves import configparser
from six.moves import input
from time import mktime

from dlrn.config import setup_logging
from dlrn.db import closeSession
from dlrn.db import Commit
from dlrn.db import getCommits
from dlrn.db import getSession
from dlrn.db import Promotion
from dlrn.utils import get_component_list

FLAG_PURGED = 0x2

logger = logging.getLogger("dlrn-purge")


def is_commit_in_dirs(commit, dirlist, basedir, component_list=None):
    if dirlist is None:
        return False
    if commit.artifacts is None:
        return False
    directories = dirlist.split(',')
    rpms = []
    for rpm in commit.artifacts.split(','):
        rpms.append(rpm.split('/')[-1])

    for rpm in rpms:
        for directory in directories:
            if os.path.exists(os.path.join(directory, rpm)):
                return True
            # If using components, search for the relative component paths too
            if component_list:
                relpath = os.path.relpath(directory, basedir)
                for component in component_list:
                    newpath = os.path.join(basedir, 'component', component,
                                           relpath)
                    if os.path.exists(os.path.join(newpath, rpm)):
                        return True
    return False


def purge_promoted_hashes(config, timestamp, dry_run=True):
    session = getSession(config.get('DEFAULT', 'database_connection'))
    basedir = os.path.join(config.get('DEFAULT', 'datadir'), 'repos')
    reponame = config.get('DEFAULT', 'reponame')

    # Get list of all promote names
    all_promotions = session.query(Promotion).\
        distinct(Promotion.promotion_name).\
        group_by(Promotion.promotion_name).all()
    closeSession(session)

    promotion_list = ['current', 'consistent']
    for prom in all_promotions:
        promotion_list.append(prom.promotion_name)

    logger.debug("Promotion list: %s" % promotion_list)

    # Now go through all directories
    for prom in promotion_list:
        directory = os.path.join(basedir, prom)
        logger.info("Looking into directory: %s" % directory)
        if os.path.islink(os.path.join(directory, reponame + '.repo')):
            protected_path = os.path.dirname(
                os.path.realpath(os.path.join(directory, reponame + '.repo')))
        else:
            logger.warning('No symlinks at %s' % directory)
            protected_path = ''

        logger.debug("Setting protected path: %s" % protected_path)
        # We have to traverse a 3-level hash structure
        # Not deleting the first two levels (xx/yy), just the final level,
        # where the files are located
        for path in glob.glob('%s/??/??/*' % directory):
            if os.path.isdir(path):
                dirstats = os.stat(path)
                if timestamp > dirstats.st_mtime:
                    if os.path.realpath(path) == protected_path:
                        logger.info('Not deleting %s, it is protected' % path)
                        continue
                    logger.info("Remove %s" % path)
                    if not dry_run:
                        shutil.rmtree(path, ignore_errors=True)


def purge():
    parser = argparse.ArgumentParser()
    # Some of the non-positional arguments are required, so change the text
    # saying "optional arguments" to just "arguments":
    parser._optionals.title = 'arguments'

    parser.add_argument('--config-file',
                        help="Config file (required)", required=True)
    parser.add_argument('--older-than',
                        help="Purge builds older than provided value"
                             " (in days).", required=True)
    parser.add_argument('-y', help="Answer \"yes\" to any questions",
                        action="store_true")
    parser.add_argument('--dry-run', help="Do not change anything, show"
                        " what changes would be made",
                        action="store_true")
    parser.add_argument('--exclude-dirs', help="Do not remove commits whose"
                        " packages are included in one of the specifided"
                        " directories (comma-separated list).")
    parser.add_argument('--debug', action='store_true',
                        help="Print debug logs")

    options = parser.parse_args(sys.argv[1:])

    setup_logging(options.debug)

    cp = configparser.RawConfigParser()
    cp.read(options.config_file)

    timeparsed = datetime.now() - timedelta(days=int(options.older_than))

    if options.y is False:
        ans = input(("Remove all data before %s, correct? [N/y] " %
                     timeparsed.ctime()))
        if ans.lower() != "y":
            return

    session = getSession(cp.get('DEFAULT', 'database_connection'))
    try:
        use_components = cp.getboolean('DEFAULT', 'use_components')
    except ValueError:
        use_components = False
    basedir = os.path.abspath(os.path.join(cp.get('DEFAULT', 'datadir'),
                                           'repos'))
    if use_components:
        component_list = get_component_list(session)
    else:
        component_list = None
    logger.debug("Used components: %s" % component_list)

    # To remove builds we have to start at a point in time and move backwards
    # builds with no build date are also purged as these are legacy
    # All repositories can have the repodata directory and symlinks purged
    # But we must keep the rpm files of the most recent successful build of
    # each project as other symlinks not being purged will be pointing to them.
    topurge = getCommits(session,
                         limit=0,
                         before=int(mktime(timeparsed.timetuple()))
                         ).all()

    logger.debug("Commmits from %s days ago: %s" % (options.older_than,
                                                    topurge))

    fullpurge = []
    for commit in topurge:
        if commit.flags & FLAG_PURGED:
            logger.debug("Commit %s was purged" % commit)
            continue

        if is_commit_in_dirs(commit, options.exclude_dirs, basedir,
                             component_list=component_list):
            # The commit RPMs are in one of the directories
            # that should not be touched.
            logger.info("Ignoring commit %s for %s, it is in one of the"
                        " excluded directories" % (commit.id,
                                                   commit.project_name))
            continue

        datadir = os.path.join(cp.get('DEFAULT', 'datadir'), "repos",
                               commit.getshardedcommitdir())
        if commit.project_name not in fullpurge and commit.status == "SUCCESS":
            # So we have not removed any commit from this project yet, and it
            # is successful. Is it the newest one?
            previouscommits = getCommits(session,
                                         project=commit.project_name,
                                         since=commit.dt_build,
                                         with_status='SUCCESS').count()

            if previouscommits == 0:
                logger.info("Keeping old commit for %s" % commit.project_name)
                continue  # this is the newest commit for this project, keep it

            try:
                for entry in os.listdir(datadir):
                    entry = os.path.join(datadir, entry)
                    if entry.endswith(".rpm") and not os.path.islink(entry):
                        logger.debug("Skipping dir or file %s" % entry)
                        continue
                    if os.path.isdir(entry):
                        logger.info("Remove %s" % entry)
                        if options.dry_run is False:
                            shutil.rmtree(entry)
                    else:
                        logger.info("Delete %s" % entry)
                        if options.dry_run is False:
                            os.unlink(entry)
            except OSError:
                logger.warning("Cannot access directory %s for purge,"
                               " ignoring." % datadir)
            fullpurge.append(commit.project_name)
            commit.flags |= FLAG_PURGED
            logger.info("Remove %s" % datadir)
            if options.dry_run is False:
                shutil.rmtree(datadir, ignore_errors=True)
        else:
            # If the commit was not successful, we need to be careful not to
            # remove the directory if there was a successful build
            if commit.status != "SUCCESS":
                othercommits = session.query(Commit).filter(
                    Commit.project_name == commit.project_name,
                    Commit.commit_hash == commit.commit_hash,
                    Commit.status == 'SUCCESS').count()

                if othercommits == 0:
                    logger.info("Remove %s" % datadir)
                    if options.dry_run is False:
                        shutil.rmtree(datadir, ignore_errors=True)
            else:
                logger.info("Remove %s" % datadir)
                if options.dry_run is False:
                    shutil.rmtree(datadir, ignore_errors=True)
            commit.flags |= FLAG_PURGED
    if options.dry_run is False:
        session.commit()
    closeSession(session)

    if cp.getboolean('DEFAULT', 'use_components'):
        purge_promoted_hashes(cp, mktime(timeparsed.timetuple()),
                              dry_run=options.dry_run)



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

import logging
import os
import sh

from dlrn.config import getConfigOptions

logger = logging.getLogger("dlrn-rsync")


def sync_repo(commit):
    config_options = getConfigOptions()
    rsyncdest = config_options.rsyncdest
    rsyncport = config_options.rsyncport
    datadir = os.path.realpath(config_options.datadir)

    if rsyncdest != '':
        # We are only rsyncing the current repo dir to rsyncdest
        rsyncpaths = []
        # We are inserting a dot in the path after repos, this is used by
        # rsync -R (see man rsync)
        commitdir_abs = os.path.join(datadir, "repos", ".",
                                     commit.getshardedcommitdir())
        rsyncpaths.append(commitdir_abs)
        # We also need report.html, status_report.html, queue.html,
        # styles.css and the consistent and current symlinks
        for filename in ['report.html', 'status_report.html', 'styles.css',
                         'queue.html', 'status_report.csv']:
            filepath = os.path.join(datadir, "repos", ".", filename)
            rsyncpaths.append(filepath)

        rsh_command = 'ssh -p %s -o StrictHostKeyChecking=no' % rsyncport
        try:
            sh.rsync('-avzR', '--delete-delay',
                     '-e', rsh_command,
                     rsyncpaths, rsyncdest)
        except Exception as e:
            logger.warn('Failed to rsync content to %s ,'
                        'got error %s' % (rsyncdest, e))
            # Raise exception, so it can be treated as an error
            raise e


def sync_symlinks(commit):
    config_options = getConfigOptions()
    rsyncdest = config_options.rsyncdest
    rsyncport = config_options.rsyncport
    datadir = os.path.realpath(config_options.datadir)
    reponame = config_options.reponame

    if rsyncdest != '':
        # We want to sync the symlinks in a second pass, once all content
        # has been copied, to avoid a race condition it they are copied first
        rsyncpaths = []
        for filename in ['consistent', 'current']:
            if config_options.use_components:
                filepath = os.path.join(datadir, "repos", ".", "component",
                                        commit.component, filename)
            else:
                filepath = os.path.join(datadir, "repos", ".", filename)
            rsyncpaths.append(filepath)

        # If using components, current and consistent on the top-level dir
        # are not symlinks, but full directories

        if config_options.use_components:
            exclude_list = []
            extra_include_list = []
            for filename in ['consistent', 'current']:
                filepath = os.path.join(datadir, "repos", ".", filename)
                rsyncpaths.append(filepath)

                exclude_list.extend(['--exclude', '%s/%s.repo' %
                                     (filename, reponame)])
                exclude_list.extend(['--exclude', '%s/%s.repo.md5' %
                                     (filename, reponame)])
                exclude_list.extend(['--exclude', '%s/versions.csv' %
                                    filename])
                extra_include_list.append('%s/%s.repo' % (filepath, reponame))
                extra_include_list.append('%s/%s.repo.md5' %
                                          (filepath, reponame))
                extra_include_list.append('%s/versions.csv' % filepath)

        rsh_command = 'ssh -p %s -o StrictHostKeyChecking=no' % rsyncport

        if config_options.use_components:
            try:
                # First, rsync everything except the top-level symlinks
                sh.rsync('-avzR', '--delete-delay',
                         '-e', rsh_command,
                         exclude_list,
                         rsyncpaths, rsyncdest)
            except Exception as e:
                # We are not raising exceptions for symlink rsyncs, these will
                # be fixed after another build
                logger.warn('Failed to rsync symlinks to %s ,'
                            'got error %s' % (rsyncdest, e))
            try:
                # Then, the top-level symlinks
                sh.rsync('-avzR', '--delete-delay',
                         '-e', rsh_command,
                         extra_include_list, rsyncdest)
            except Exception as e:
                # We are not raising exceptions for symlink rsyncs, these will
                # be fixed after another build
                logger.warn('Failed to rsync symlinks to %s ,'
                            'got error %s' % (rsyncdest, e))
        else:
            try:
                sh.rsync('-avzR', '--delete-delay',
                         '-e', rsh_command,
                         rsyncpaths, rsyncdest)
            except Exception as e:
                # We are not raising exceptions for symlink rsyncs, these will
                # be fixed after another build
                logger.warn('Failed to rsync symlinks to %s ,'
                            'got error %s' % (rsyncdest, e))



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.
import logging
import os
import re

from dlrn.utils import import_class

_config_options = None


def _default_datadir():
    return os.path.realpath(
        os.path.join(os.path.dirname(os.path.abspath(__file__)),
                     "../data"))


def _default_scriptsdir():
    return os.path.realpath(
        os.path.join(os.path.dirname(os.path.abspath(__file__)),
                     "../scripts"))


def _default_templatedir():
    return os.path.join(
        os.path.dirname(os.path.abspath(__file__)), "templates")


DLRN_CORE_CONFIG = {
    'DEFAULT': {
        'tags': {},
        'datadir': {'default': _default_datadir()},
        'gerrit': {},
        'maxretries': {'type': 'int', 'default': 3},
        'baseurl': {},
        'smtpserver': {},
        'distro': {},
        'source': {},
        'target': {},
        'reponame': {},
        'rsyncdest': {'default': ''},
        'rsyncport': {'default': 22},
        'scriptsdir': {'default': _default_scriptsdir()},
        'configdir': {},
        'templatedir': {'default': _default_templatedir()},
        'project_name': {'default': 'RDO'},
        'pkginfo_driver': {'default': 'dlrn.drivers.rdoinfo.RdoInfoDriver'},
        'build_driver': {'default': 'dlrn.drivers.mockdriver.MockBuildDriver'},
        'workers': {'type': 'int', 'default': 1},
        'gerrit_topic': {'default': 'rdo-FTBFS'},
        'database_connection': {'default': 'sqlite:///commits.sqlite'},
        'fallback_to_master': {'type': 'boolean', 'default': True},
        'nonfallback_branches': {'type': 'list',
                                 'default': ['^master$', '^rpm-master$']},
        'release_numbering': {'default': '0.date.hash'},
        'release_minor':     {'default': '0'},
        'custom_preprocess': {'type': 'list'},
        'include_srpm_in_repo': {'type': 'boolean', 'default': True},
        'keep_changelog': {'type': 'boolean', 'default': False},
        'allow_force_rechecks': {'type': 'boolean', 'default': False},
        'use_components': {'type': 'boolean', 'default': False},
    }
}


def setup_logging(debug=False, filename=None):
    logging.basicConfig(
        level=logging.DEBUG if debug else logging.INFO,
        filename=filename,
        format='%(asctime)s %(levelname)s:%(name)s:%(message)s')
    # Silence sh module logs
    logging.getLogger("sh.command").setLevel(logging.CRITICAL)


class ConfigOptions(object):

    def __init__(self, cp, overrides=None):
        self.parse_overrides(cp, overrides)
        self.parse_config(DLRN_CORE_CONFIG, cp)
        # dynamic directory defaults
        if not self.configdir:
            self.configdir = self.scriptsdir
        if not self.templatedir:
            self.templatedir = \
                os.path.join(os.path.dirname(os.path.realpath(__file__)),
                             "templates")

        # handling for optional sections, driver-based
        drivers = [
            self.pkginfo_driver,
            self.build_driver,
        ]
        for d in drivers:
            # import driver specific config options
            driver = import_class(d)
            try:
                driver_cfg = driver.DRIVER_CONFIG
            except AttributeError:
                driver_cfg = None
            if driver_cfg:
                self.parse_config(driver_cfg, cp)

        global _config_options
        _config_options = self

    def parse_overrides(self, config_parser, overrides):
        if overrides is None:
            return
        # Now check for any potential overrides
        for rule in overrides:
            rule_match = re.match(r'^(\w+)\.(\w+)=(\w+)$', rule)
            if rule_match is not None:
                section = rule_match.group(1)
                key = rule_match.group(2)
                value = rule_match.group(3)
                if section == 'DEFAULT' or config_parser.has_section(section):
                    if config_parser.has_option(section, key):
                        logging.info("Overriding option %s.%s with value %s" %
                                     (section, key, value))
                        config_parser.set(section, key, value)
                    else:
                        logging.error("Option %s.%s is not present in the"
                                      "configuration file." % (section, key))
                        raise RuntimeError("Unknown config option %s.%s" %
                                           (section, key))

    def parse_config(self, config_rules, config_parser):
        for section, rules in config_rules.items():
            if section == 'DEFAULT' or config_parser.has_section(section):
                for option, rule in rules.items():
                    ini_option = rule.get('name', option)
                    if config_parser.has_option(section, ini_option):
                        _type = rule.get('type', '')
                        if _type == 'list' or _type == 'str':
                            cp_get_method_name = 'get'
                        else:
                            cp_get_method_name = 'get' + _type
                        cp_get_method = getattr(config_parser,
                                                cp_get_method_name)
                        val = cp_get_method(section, ini_option)
                        if _type == 'list':
                            # comma separated list
                            val = val.split(',')
                        setattr(self, option, val)
                    else:
                        self.set_default(option, rule)
            else:
                # section is missing, fill in defaults
                for option, rule in rules.items():
                    self.set_default(option, rule)

    def set_default(self, option, rule):
        if rule.get('ignore_missing'):
            # ignore_missing prevents setting default value
            return
        if 'default' in rule:
            val = rule['default']
        else:
            defaults = {
                'boolean': False,
                'int': 0,
                'str': '',
                'list': [],
            }
            val = None
            _type = rule.get('type')
            if _type and _type in defaults:
                val = defaults[_type]
        setattr(self, option, val)


def getConfigOptions():
    return _config_options



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.


class BuildRPMDriver(object):
    """Abstract base class for Build RPM drivers

    """
    def __init__(self, *args, **kwargs):
        self.config_options = kwargs.get('cfg_options')

    def build_package(self):
        """Perform package build

        """
        return False



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

# The RdoInfoDriver provides the following:
#
# 1- A getpackages function based on output provided by rdoinfo
#    (https://github.com/redhat-openstack/rdoinfo)
#
# 2- A getinfo function based on a multi-distgit repo paradigm
#
# 3- A preprocess function that is only used if the distgit directory
#    containts a .spec.j2 file

from dlrn.db import Commit
from dlrn.drivers.pkginfo import PkgInfoDriver
from dlrn.repositories import getdistrobranch
from dlrn.repositories import getsourcebranch
from dlrn.repositories import refreshrepo
from dlrn.utils import run_external_preprocess

import logging
import os
import sh

from distroinfo import query
from distroinfo import info


logger = logging.getLogger("dlrn-rdoinfo-driver")
rdoinfo_repo = ('https://raw.githubusercontent.com/'
                'redhat-openstack/rdoinfo/master/')

def buildtagsonly(package):
    return ('tags' in package and package['tags'] is not None and
            'build-tags-only' in package['tags'] or
            'build-tags-only' in package)


class RdoInfoDriver(PkgInfoDriver):
    DRIVER_CONFIG = {
        'rdoinfo_driver': {
            'rdoinfo_repo': {'name': 'repo'},
            'rdoinfo_file': {'name': 'info_files', 'type': 'list',
                             'default': ['rdo.yml']},
            'cache_dir': {},
        }
    }

    def __init__(self, *args, **kwargs):
        super(RdoInfoDriver, self).__init__(*args, **kwargs)
        self.distroinfo_path = None

    def getpackages(self, **kwargs):
        """ Valid parameters:
        :param local_info_repo: local rdoinfo repo to use instead of fetching
                                the default one using rdopkg.
        :param tags: OpenStack release tags to use (mitaka, newton, etc).
        """
        local_info_repo = kwargs.get('local_info_repo')
        tags = kwargs.get('tags')
        inforepo = None
        info_files = self.config_options.rdoinfo_file

        if local_info_repo:
            inforepo = info.DistroInfo(
                info_files=self.config_options.rdoinfo_file,
                local_info=local_info_repo,
                cache_base_path=self.config_options.cache_dir)
            # NOTE(jpena): in general, info_files will only contain one file,
            # but it supports multiple... In that case, we will have a comma
            # separated list of URLs
            self.distroinfo_path = "%s/%s" % (local_info_repo.rstrip('/'),
                                              info_files[0])
            for extra_file in info_files[1:]:
                self.distroinfo_path += ",%s/%s" % (
                    local_info_repo.rstrip('/'))
        elif self.config_options.rdoinfo_repo:
            inforepo = info.DistroInfo(
                info_files=self.config_options.rdoinfo_file,
                remote_git_info=self.config_options.rdoinfo_repo,
                cache_base_path=self.config_options.cache_dir)
            self.distroinfo_path = "%s/%s" % (
                self.config_options.rdoinfo_repo.rstrip('/'), info_files[0])
            for extra_file in info_files[1:]:
                self.distroinfo_path += ",%s/%s" % (
                    self.config_options.rdoinfo_repo.rstrip('/'))
        else:
            # distroinfo will fetch info files from the rdoinfo repo as needed
            # and store them under ~/.distroinfo/cache
            inforepo = info.DistroInfo(
                info_files=self.config_options.rdoinfo_file,
                remote_info=rdoinfo_repo,
                cache_base_path=self.config_options.cache_dir)
            self.distroinfo_path = "%s/%s" % (rdoinfo_repo.rstrip('/'),
                                              info_files[0])
            for extra_file in info_files[1:]:
                self.distroinfo_path += ",%s/%s" % (
                    rdoinfo_repo.rstrip('/'))

        pkginfo = inforepo.get_info(apply_tag=tags)

        self.packages = pkginfo["packages"]
        if tags:
            # FIXME allow list of tags?
            self.packages = query.filter_pkgs(self.packages, {'tags': tags})
        return self.packages

    def getinfo(self, **kwargs):
        project = kwargs.get('project')
        package = kwargs.get('package')
        since = kwargs.get('since')
        local = kwargs.get('local')
        dev_mode = kwargs.get('dev_mode')
        datadir = self.config_options.datadir
        repo = package['upstream']
        distro = package['master-distgit']
        tags_only = buildtagsonly(package)

        distro_dir = self._distgit_clone_dir(package['name'])
        distro_dir_full = self.distgit_dir(package['name'])
        distro_branch = getdistrobranch(package)
        source_branch = getsourcebranch(package)

        if dev_mode is False:
            try:
                distro_branch, distro_hash, dt_distro = refreshrepo(
                    distro, distro_dir, distro_branch, local=local,
                    full_path=distro_dir_full)
            except Exception:
                # The error was already logged by refreshrepo, and we want
                # to avoid halting the whole run because this distgit repo
                # failed, so return an empty list
                return []
        else:
            distro_hash = "dev"
            dt_distro = 0  # Doesn't get used in dev mode
            if not os.path.isdir(distro_dir):
                # We should fail in this case, since we are running
                # in dev mode, so no try/except
                refreshrepo(distro, distro_dir, distro_branch, local=local,
                            full_path=distro_dir_full)

        # repo is usually a string, but if it contains more then one entry we
        # git clone into a project subdirectory
        repos = [repo]
        if isinstance(repo, list):
            repos = repo
        project_toprocess = []
        for repo in repos:
            repo_dir = os.path.join(datadir, project)
            if len(repos) > 1:
                repo_dir = os.path.join(repo_dir, os.path.split(repo)[1])
            try:
                source_branch, _, _ = refreshrepo(repo, repo_dir, source_branch,
                                                  local=local)
            except Exception:
                # The error was already logged by refreshrepo, and the only
                # side-effect is that we are not adding this commit to the
                # list of commits to be processed, so we can ignore it and
                # move on to the next repo
                continue

            git = sh.git.bake(_cwd=repo_dir, _tty_out=False)
            # Git gives us commits already sorted in the right order
            if tags_only is True:
                logger.info('Building tags only for %s' % project)
                if since == '-1':
                    # we need 2 entries as HEAD will be listed too
                    since = '-2'
                lines = filter(
                    lambda x: x.find('tag: ') >= 0,
                    git.log('--simplify-by-decoration',
                            "--pretty=format:'%ct %H %d'",
                            since, "--first-parent",
                            "--reverse", "%s" % source_branch))
            else:
                lines = git.log("--pretty=format:'%ct %H'",
                                since, "--first-parent",
                                "--reverse")

            for line in lines:
                dt, commit_hash = str(line).strip().strip("'").split(" ")[:2]
                if self.config_options.use_components and 'component' in package:
                    component = package['component']
                else:
                    component = None
                commit = Commit(dt_commit=float(dt), project_name=project,
                                type="rpm",
                                commit_hash=commit_hash, repo_dir=repo_dir,
                                distro_hash=distro_hash, dt_distro=dt_distro,
                                distgit_dir=self.distgit_dir(package['name']),
                                commit_branch=source_branch,
                                dt_extended=0, extended_hash=None,
                                component=component)
                project_toprocess.append(commit)

        return project_toprocess

    def preprocess(self, **kwargs):
        # Pre-processing is only required if we have a jinja2 spec template
        package_name = kwargs.get('package_name')
        commit_hash = kwargs.get('commit_hash')
        distgit_dir = self.distgit_dir(package_name)
        source_dir = "%s/%s" % (self.config_options.datadir, package_name)
        # Now, try to check if we need to run a pre-processing job
        preprocess_needed = False
        for f in os.listdir(distgit_dir):
            if f.endswith('.spec.j2'):
                # We have a template here, so we have to preprocess
                preprocess_needed = True
                break
        if preprocess_needed:
            logger.info('Pre-processing template at %s' % distgit_dir)
            renderspec = sh.renderspec.bake(_cwd=distgit_dir,
                                            _tty_out=False, _timeout=3600)
            renderspec('--spec-style', 'fedora', '--epoch',
                       '../../epoch/fedora.yaml')

        for custom_preprocess in self.config_options.custom_preprocess:
            if custom_preprocess != '':
                run_external_preprocess(
                    cmdline=custom_preprocess,
                    pkgname=package_name,
                    distgit=distgit_dir,
                    distroinfo=self.distroinfo_path,
                    source_dir=source_dir,
                    commit_hash=commit_hash)
        return

    def distgit_dir(self, package_name):
        datadir = self.config_options.datadir
        # Find extra directory inside it, if needed
        extra_dir = '/'
        for package in self.packages:
            if package['name'] == package_name:
                if 'distgit-path' in package:
                    extra_dir = package['distgit-path']
                    break
        return os.path.join(datadir, package_name + "_distro",
                            extra_dir.lstrip('/'))

    def _distgit_clone_dir(self, package_name):
        datadir = self.config_options.datadir
        return os.path.join(datadir, package_name + "_distro")



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.
from dlrn.api import app
from dlrn.db import getSession
from dlrn.db import User
from flask_httpauth import HTTPBasicAuth

import passlib.hash

auth = HTTPBasicAuth()


@auth.verify_password
def verify_pw(username, password):
    session = getSession(app.config['DB_PATH'])
    user = session.query(User).filter(User.username == username).first()
    if user is not None:
        return passlib.hash.sha512_crypt.verify(password, user.password)
    else:
        return False


class InvalidUsage(Exception):
    status_code = 400

    def __init__(self, message, status_code=None, payload=None):
        Exception.__init__(self)
        self.message = message
        if status_code is not None:
            self.status_code = status_code
        self.payload = payload

    def to_dict(self):
        rv = dict(self.payload or ())
        rv['message'] = self.message
        return rv


class RepoDetail(object):
    def __init__(self):
        self.commit_hash = None
        self.distro_hash = None
        self.distro_hash_short = None
        self.success = 0
        self.failure = 0
        self.timestamp = 0
        self.component = None


class AggDetail(object):
    def __init__(self):
        self.ref_hash = None
        self.success = 0
        self.failure = 0
        self.timestamp = 0
